@@ -7,7 +7,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: common.cpp 3195 2006-02-27 19:38:05Z mosu $
+   $Id: common.cpp 3455 2007-01-10 13:13:23Z mosu $
 
    helper functions, common variables
 
@@ -16,7 +16,7 @@
 
 #include "os.h"
 
-#include <assert.h>
+#include <cassert>
 #include <ctype.h>
 #include <errno.h>
 #include <iconv.h>
@@ -658,7 +658,7 @@ remove_unique_uint32(uint32_t number,
 
   if (hack_engaged(ENGAGE_NO_VARIABLE_DATA))
     return true;
-  foreach(dit, ru_numbers[category])
+  mxforeach(dit, ru_numbers[category])
     if (*dit == number) {
       ru_numbers[category].erase(dit);
       return true;
