@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: common.h 3465 2007-01-13 18:08:21Z mosu $
+   $Id: common.h 3496 2007-02-21 10:29:30Z mosu $
 
    definitions used in all programs, helper functions
 
@@ -55,7 +55,7 @@ namespace libebml {
 
 using namespace libebml;
 
-#define VERSIONNAME "After The Rain Has Fallen"
+#define VERSIONNAME "You're My Flame"
 #define VERSIONINFO "mkvmerge v" VERSION " ('" VERSIONNAME "')"
 #define BUGMSG _("This should not have happened. Please contact the author " \
                  "Moritz Bunkus <moritz@bunkus.org> with this error/warning " \
@@ -294,7 +294,7 @@ extern int MTX_DLL_API verbose;
 #define mxforeach(it, vec) for (it = (vec).begin(); it != (vec).end(); it++)
 #define mxfind(value, cont) std::find(cont.begin(), cont.end(), value)
 #define mxfind2(it, value, cont) \
-  ((id = std::find((cont).begin(), (cont).end(), value)) != (cont).end())
+  ((it = std::find((cont).begin(), (cont).end(), value)) != (cont).end())
 #define map_has_key(m, k) ((m).end() != (m).find(k))
 
 class MTX_DLL_API bitvalue_c {
