@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: compression.cpp 3155 2005-12-02 21:39:26Z mosu $
+   $Id: compression.cpp 3377 2006-11-23 15:56:44Z mosu $
 
    Helper routines for various compression libs
 
@@ -573,7 +573,7 @@ content_decoder_c::reverse(memory_cptr &memory,
   if (!is_ok() || encodings.empty())
     return;
 
-  foreach(ce, encodings)
+  mxforeach(ce, encodings)
     if (0 != (ce->scope & scope))
       ce->compressor->decompress(memory);
 }
