@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: hacks.cpp 3358 2006-11-10 18:23:51Z mosu $
+   $Id: hacks.cpp 3493 2007-02-19 08:52:27Z mosu $
 
    hacks :)
 
@@ -41,6 +41,7 @@ static const char *mosu_hacks[] = {
   ENGAGE_KEEP_BITSTREAM_AR_INFO,
   ENGAGE_USE_SIMPLE_BLOCK,
   ENGAGE_OLD_AAC_CODECID,
+  ENGAGE_USE_CODEC_STATE,
   NULL
 };
 static vector<string> engaged_hacks;
@@ -49,7 +50,7 @@ bool
 hack_engaged(const string &hack) {
   vector<string>::const_iterator hit;
 
-  foreach(hit, engaged_hacks)
+  mxforeach(hit, engaged_hacks)
     if (*hit == hack)
       return true;
 
