@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: iso639.cpp 3160 2005-12-06 16:17:01Z mosu $
+   $Id: iso639.cpp 3166 2005-12-10 11:03:17Z mosu $
 
    ISO639 language definitions, lookup functions
 
@@ -574,7 +574,7 @@ is_popular_language(const char *lang) {
 bool
 is_popular_language_code(const char *code) {
   return
-    !strcmp(code, "zho") || // Chinese
+    !strcmp(code, "chi") || // Chinese
     !strcmp(code, "dut") || // Dutch
     !strcmp(code, "eng") || // English
     !strcmp(code, "fin") || // Finnish
