@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: mm_io.cpp 3152 2005-12-01 21:16:12Z mosu $
+   $Id: mm_io.cpp 3198 2006-03-02 19:52:19Z mosu $
 
    IO callback class implementation
 
@@ -663,6 +663,20 @@ mm_io_c::read_uint64_be() {
   return get_uint64_be(buffer);
 }
 
+uint32_t
+mm_io_c::read(memory_cptr &buffer,
+              size_t size,
+              int offset) {
+  if (-1 == offset)
+    offset = buffer->get_size();
+  if (buffer->get_size() <= (size + offset))
+    buffer->resize(size + offset);
+  if (read(buffer->get() + offset, size) != size)
+    throw mm_io_eof_error_c();
+  buffer->set_size(size + offset);
+  return size;
+}
+
 int
 mm_io_c::write_uint8(unsigned char value) {
   return write(&value, 1);
@@ -716,6 +730,17 @@ mm_io_c::write_uint64_be(uint64_t value) {
   return write(&buffer, sizeof(uint64_t));
 }
 
+uint32_t
+mm_io_c::write(const memory_cptr &buffer,
+               int size,
+               int offset) {
+  if (-1 == size)
+    size = buffer->get_size();
+  if (write(buffer->get() + offset, size) != size)
+    throw mm_io_eof_error_c();
+  return size;
+}
+
 void
 mm_io_c::skip(int64 num_bytes) {
   int64_t pos;
@@ -1022,6 +1047,12 @@ mm_mem_io_c::eof() {
   return pos >= mem_size;
 }
 
+unsigned char *
+mm_mem_io_c::get_and_lock_buffer() {
+  free_mem = false;
+  return mem;
+}
+
 /*
    Class for handling UTF-8/UTF-16/UTF-32 text files.
 */
