@@ -5,7 +5,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: tracks.cpp 3128 2005-10-21 11:34:12Z mosu $
+   $Id: tracks.cpp 3469 2007-01-15 19:02:18Z mosu $
 
    extracts tracks from Matroska files into other files
 
@@ -23,7 +23,7 @@
 #include <time.h>
 
 #if defined(COMP_MSC)
-#include <assert.h>
+#include <cassert>
 #else
 #include <unistd.h>
 #endif
@@ -135,8 +135,9 @@ create_extractors(KaxTracks &kax_tracks,
     // We're done.
     extractors.push_back(extractor);
 
-    mxinfo("Extracting track " LLD " with the CodecID '%s' to the file '%s'."
-           "\n", tnum, codec_id.c_str(), tspec->out_name);
+    mxinfo("Extracting track " LLD " with the CodecID '%s' to the file '%s'. "
+           "Container format: %s\n", tnum, codec_id.c_str(), tspec->out_name,
+           extractor->get_container_name());
   }
 
   // Signal that all headers have been taken care of.
@@ -152,6 +153,7 @@ handle_blockgroup(KaxBlockGroup &blockgroup,
   KaxBlockDuration *kduration;
   KaxReferenceBlock *kreference;
   KaxBlockAdditions *kadditions;
+  KaxCodecState *kcstate;
   xtr_base_c *extractor;
   int64_t duration, fref, bref;
   int i;
@@ -198,6 +200,13 @@ handle_blockgroup(KaxBlockGroup &blockgroup,
   if (0 > duration)
     duration = extractor->default_duration * block->NumberFrames();
 
+  kcstate = FINDFIRST(&blockgroup, KaxCodecState);
+  if (NULL != kcstate) {
+    memory_cptr codec_state(new memory_c(kcstate->GetBuffer(),
+                                         kcstate->GetSize(), false));
+    extractor->handle_codec_state(codec_state);
+  }
+
   for (i = 0; i < block->NumberFrames(); i++) {
     int64_t this_timecode, this_duration;
 
