@@ -5,7 +5,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: xtr_aac.h 3126 2005-10-19 16:52:55Z mosu $
+   $Id: xtr_aac.h 3461 2007-01-12 14:44:59Z mosu $
 
    extracts tracks from Matroska files into other files
 
@@ -31,6 +31,10 @@ public:
                             int64_t timecode, int64_t duration, int64_t bref,
                             int64_t fref, bool keyframe, bool discardable,
                             bool references_valid);
+
+  virtual const char *get_container_name() {
+    return "raw AAC file with ADTS headers";
+  };
 };
 
 #endif
