@@ -5,7 +5,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: xtr_avc.h 3126 2005-10-19 16:52:55Z mosu $
+   $Id: xtr_avc.h 3461 2007-01-12 14:44:59Z mosu $
 
    extracts tracks from Matroska files into other files
 
@@ -33,6 +33,10 @@ public:
                             bool references_valid);
   void write_nal(const binary *data, int &pos, int data_size,
                  int nal_size_size);
+
+  virtual const char *get_container_name() {
+    return "AVC/h.264 elementary stream";
+  };
 };
 
 #endif
