@@ -5,7 +5,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: xtr_base.cpp 3461 2007-01-12 14:44:59Z mosu $
+   $Id: xtr_base.cpp 3469 2007-01-15 19:02:18Z mosu $
 
    extracts tracks from Matroska files into other files
 
@@ -176,3 +176,8 @@ xtr_fullraw_c::create_file(xtr_base_c *_master,
   if ((NULL != priv) && (0 != priv->GetSize()))
     out->write(priv->GetBuffer(), priv->GetSize());
 }
+
+void
+xtr_fullraw_c::handle_codec_state(memory_cptr &codec_state) {
+  out->write(codec_state->get(), codec_state->get_size());
+}
