@@ -5,7 +5,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: xtr_cpic.h 3177 2006-02-03 09:25:30Z mosu $
+   $Id: xtr_cpic.h 3461 2007-01-12 14:44:59Z mosu $
 
    extracts tracks from Matroska files into other files
 
@@ -32,6 +32,10 @@ public:
                             int64_t timecode, int64_t duration, int64_t bref,
                             int64_t fref, bool keyframe, bool discardable,
                             bool references_valid);
+
+  virtual const char *get_container_name() {
+    return "CorePicture/CorePanorama";
+  };
 };
 
 #endif
