@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_avi.cpp 3464 2007-01-13 17:41:08Z mosu $
+   $Id: r_avi.cpp 3493 2007-02-19 08:52:27Z mosu $
 
    AVI demultiplexer module
 
@@ -31,6 +31,7 @@ extern "C" {
 #include "aac_common.h"
 #include "common.h"
 #include "error.h"
+#include "hacks.h"
 #include "mpeg4_common.h"
 #include "output_control.h"
 #include "r_avi.h"
@@ -170,7 +171,8 @@ avi_reader_c::create_packetizer(int64_t tid) {
       if (verbose)
         mxinfo(FMT_TID "Using the MPEG-4 part 2 video output module.\n",
                ti.fname.c_str(), (int64_t)0);
-    } else if (mpeg4::p10::is_avc_fourcc(codec)) {
+    } else if (mpeg4::p10::is_avc_fourcc(codec) &&
+               !hack_engaged(ENGAGE_ALLOW_AVC_IN_VFW_MODE)) {
       try {
         memory_cptr avcc = extract_avcc();
         mpeg4_p10_es_video_packetizer_c *ptzr =
@@ -596,8 +598,15 @@ avi_reader_c::identify() {
       safefree(buffer);
     }
   }
-  if (!extended_info_shown)
-    mxinfo("Track ID 0: video (%s)\n", AVI_video_compressor(avi));
+  if (!extended_info_shown) {
+    string info;
+
+    if (identify_verbose && mpeg4::p10::is_avc_fourcc(type))
+      info = " [uses_avc_es_packetizer]";
+
+    mxinfo("Track ID 0: video (%s)%s\n", AVI_video_compressor(avi),
+           info.c_str());
+  }
   for (i = 0; i < AVI_audio_tracks(avi); i++) {
     AVI_set_audio_track(avi, i);
     switch (AVI_audio_format(avi)) {
