@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_matroska.cpp 3431 2007-01-05 11:15:51Z mosu $
+   $Id: r_matroska.cpp 3493 2007-02-19 08:52:27Z mosu $
 
    Matroska reader
 
@@ -1434,8 +1434,9 @@ kax_reader_c::create_packetizer(int64_t tid) {
     switch (t->type) {
       case 'v':
         if ((t->codec_id == MKV_V_MSCOMP) &&
-            mpeg4::p10::is_avc_fourcc(t->v_fourcc))
-          create_mpeg4_p10_es_video_packetizer(t);
+            mpeg4::p10::is_avc_fourcc(t->v_fourcc) &&
+            !hack_engaged(ENGAGE_ALLOW_AVC_IN_VFW_MODE))
+          create_mpeg4_p10_es_video_packetizer(t, nti);
 
         else if (starts_with(t->codec_id, "V_MPEG4", 7) ||
                  (t->codec_id == MKV_V_MSCOMP) ||
@@ -1757,7 +1758,8 @@ kax_reader_c::create_packetizers() {
 }
 
 void
-kax_reader_c::create_mpeg4_p10_es_video_packetizer(kax_track_t *t) {
+kax_reader_c::create_mpeg4_p10_es_video_packetizer(kax_track_t *t,
+                                                   track_info_c &nti) {
   try {
     read_first_frame(t);
     if (NULL == t->first_frame_data.get())
@@ -1785,7 +1787,7 @@ kax_reader_c::create_mpeg4_p10_es_video_packetizer(kax_track_t *t) {
 
     mpeg4_p10_es_video_packetizer_c *ptzr =
       new mpeg4_p10_es_video_packetizer_c(this, avcc, t->v_width, t->v_height,
-                                          ti);
+                                          nti);
     t->ptzr = add_packetizer(ptzr);
     ptzr->enable_timecode_generation(false);
     if (t->default_duration)
@@ -1953,6 +1955,7 @@ kax_reader_c::read(generic_packetizer_c *,
   KaxReferenceBlock *ref_block;
   KaxBlockDuration *duration;
   KaxClusterTimecode *ctc;
+  KaxCodecState *codec_state;
   int64_t block_fref, block_bref, block_duration, frame_duration;
   uint64_t blockadd_id;
   kax_track_t *block_track;
@@ -2186,6 +2189,15 @@ kax_reader_c::read(generic_packetizer_c *,
             if (appending)
               last_timecode -= first_timecode;
 
+            codec_state = static_cast<KaxCodecState *>
+              (block_group->FindFirstElt(KaxCodecState::ClassInfos));
+            if ((NULL != codec_state) && !hack_engaged(ENGAGE_USE_CODEC_STATE))
+              mxerror(FMT_TID "This track uses a Matroska feature called "
+                      "'Codec state elements'. mkvmerge supports these but "
+                      "this feature has not been turned on with the option "
+                      "'--engage use_codec_state'.\n", ti.fname.c_str(),
+                      (int64_t)block_track->tnum);
+
             if ((block_track->ptzr != -1) && block_track->passthrough) {
               // The handling for passthrough is a bit different. We don't have
               // any special cases, e.g. 0 terminating a string for the subs
@@ -2203,6 +2215,9 @@ kax_reader_c::read(generic_packetizer_c *,
                                last_timecode + i * frame_duration,
                                block_duration, block_bref, block_fref);
                 packet->duration_mandatory = duration != NULL;
+                if (NULL != codec_state)
+                  packet->codec_state = clone_memory(codec_state->GetBuffer(),
+                                                     codec_state->GetSize());
                 ((passthrough_packetizer_c *)PTZR(block_track->ptzr))->
                   process(packet_cptr(packet));
               }
@@ -2234,11 +2249,17 @@ kax_reader_c::read(generic_packetizer_c *,
                     lines = (char *)safemalloc(data->get_size() + 1);
                     lines[data->get_size()] = 0;
                     memcpy(lines, data->get(), data->get_size());
-                    PTZR(block_track->ptzr)->
-                      process(new packet_t(new memory_c((unsigned char *)lines,
-                                                        0, true),
-                                           last_timecode, block_duration,
-                                           block_bref, block_fref));
+
+                    packet_t *packet =
+                      new packet_t(new memory_c((unsigned char *)lines, 0,
+                                                true),
+                                   last_timecode, block_duration,
+                                   block_bref, block_fref);
+                    if (NULL != codec_state)
+                      packet->codec_state =
+                        clone_memory(codec_state->GetBuffer(),
+                                     codec_state->GetSize());
+                    PTZR(block_track->ptzr)->process(packet);
                   }
                 } else {
                   packet_cptr packet(new packet_t(data, last_timecode + i *
@@ -2246,6 +2267,10 @@ kax_reader_c::read(generic_packetizer_c *,
                                                   block_duration, block_bref,
                                                   block_fref));
 
+                  if (NULL != codec_state)
+                    packet->codec_state =
+                      clone_memory(codec_state->GetBuffer(),
+                                   codec_state->GetSize());
                   if (blockadd) {
                     for (i = 0; i < blockadd->ListSize(); i++) {
                       if (!(is_id((*blockadd)[i], KaxBlockMore)))
@@ -2264,11 +2289,9 @@ kax_reader_c::read(generic_packetizer_c *,
                         reverse(blockadded, CONTENT_ENCODING_SCOPE_BLOCK);
                       packet->data_adds.push_back(blockadded);
                     }
+                  }
 
-                    PTZR(block_track->ptzr)->process(packet);
-
-                  } else
-                    PTZR(block_track->ptzr)->process(packet);
+                  PTZR(block_track->ptzr)->process(packet);
                 }
               }
 
@@ -2389,6 +2412,11 @@ kax_reader_c::identify() {
           info += mxsprintf("stereo_mode:%d ", (int)tracks[i]->v_stereo_mode);
         info += mxsprintf("default_track:%u ",
                           tracks[i]->default_track ? 1 : 0);
+
+        if ((tracks[i]->codec_id == MKV_V_MSCOMP) &&
+            mpeg4::p10::is_avc_fourcc(tracks[i]->v_fourcc))
+          info += "uses_avc_es_packetizer ";
+
         info += "]";
       } else
         info = "";
