@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_ogm.cpp 3331 2006-10-25 14:03:51Z mosu $
+   $Id: r_ogm.cpp 3405 2007-01-01 14:28:27Z mosu $
 
    OGG media stream reader
 
@@ -446,7 +446,7 @@ ogm_reader_c::create_packetizer(int64_t tid) {
         fps = (double)10000000.0 / get_uint64_le(&sth->time_unit);
         width = get_uint32_le(&sth->sh.video.width);
         height = get_uint32_le(&sth->sh.video.height);
-        if (is_mpeg4_p2_fourcc(sth->subtype)) {
+        if (mpeg4::p2::is_fourcc(sth->subtype)) {
           ptzr = new mpeg4_p2_video_packetizer_c(this, fps, width, height,
                                                  false, ti);
           mxinfo(FMT_TID "Using the MPEG-4 part 2 video output module.\n",
