@@ -6,18 +6,21 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_ssa.cpp 3213 2006-04-28 11:33:27Z mosu $
+   $Id: r_ssa.cpp 3333 2006-11-03 15:41:47Z mosu $
 
    SSA/ASS subtitle parser
 
    Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
+#include "os.h"
+
 #include <ctype.h>
+#include <errno.h>
+#include <pcrecpp.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
-#include <errno.h>
 
 #include <algorithm>
 
@@ -34,17 +37,36 @@ int
 ssa_reader_c::probe_file(mm_text_io_c *io,
                          int64_t size) {
   string line;
+  int line_number;
+  pcrecpp::RE comment_re("^\\s*;");
 
   try {
+    line_number = 0;
     io->setFilePointer(0, seek_beginning);
-    line = io->getline();
-    if (strcasecmp(line.c_str(), "[script info]"))
+    while (io->getline2(line)) {
+      ++line_number;
+
+      strip(line, true);
+      // This is the line mkvmerge is looking for: positive match.
+      if (!strcasecmp(line.c_str(), "[script info]")) {
+        io->setFilePointer(0, seek_beginning);
+        return 1;
+      }
+
+      // Read at most 500 lines.
+      if (line_number > 500)
+        return 0;
+
+      // Allow for empty lines and comments.
+      if (comment_re.PartialMatch(line) || (line == ""))
+        continue;
+
+      // It's something else: negative match.
       return 0;
-    io->setFilePointer(0, seek_beginning);
+    }
   } catch (...) {
-    return 0;
   }
-  return 1;
+  return 0;
 }
 
 ssa_reader_c::ssa_reader_c(track_info_c &_ti)
@@ -162,6 +184,14 @@ ssa_reader_c::parse_file(mm_text_io_c *io) {
   vector<string> fields;
   ssa_section_e section, previous_section;
   bool add_to_global;
+  pcrecpp::RE_Options opt_caseless;
+  opt_caseless.set_caseless(true);
+  pcrecpp::RE sec_styles_ass_re("^\\s*\\[V4\\+\\s+Styles\\]", opt_caseless);
+  pcrecpp::RE sec_styles_re("^\\s*\\[V4\\s+Styles\\]", opt_caseless);
+  pcrecpp::RE sec_info_re("^\\s*\\[Script\\s+Info\\]", opt_caseless);
+  pcrecpp::RE sec_events_re("^\\s*\\[Events\\]", opt_caseless);
+  pcrecpp::RE sec_graphics_re("^\\s*\\[Graphics\\]", opt_caseless);
+  pcrecpp::RE sec_fonts_re("^\\s*\\[Fonts\\]", opt_caseless);
 
   num = 0;
 
@@ -179,24 +209,24 @@ ssa_reader_c::parse_file(mm_text_io_c *io) {
     if (!strcasecmp(line.c_str(), "ScriptType: v4.00+"))
       m_is_ass = true;
 
-    else if (!strcasecmp(line.c_str(), "[V4+ Styles]")) {
+    else if (sec_styles_ass_re.PartialMatch(line)) {
       m_is_ass = true;
       section = SSA_SECTION_V4STYLES;
 
-    } else if (!strcasecmp(line.c_str(), "[V4 Styles]"))
+    } else if (sec_styles_re.PartialMatch(line))
       section = SSA_SECTION_V4STYLES;
 
-    else if (!strcasecmp(line.c_str(), "[Script Info]"))
+    else if (sec_info_re.PartialMatch(line))
       section = SSA_SECTION_INFO;
 
-    else if (!strcasecmp(line.c_str(), "[Events]"))
+    else if (sec_events_re.PartialMatch(line))
       section = SSA_SECTION_EVENTS;
 
-    else if (!strcasecmp(line.c_str(), "[Graphics]")) {
+    else if (sec_graphics_re.PartialMatch(line)) {
       section = SSA_SECTION_GRAPHICS;
       add_to_global = false;
 
-    } else if (!strcasecmp(line.c_str(), "[Fonts]")) {
+    } else if (sec_fonts_re.PartialMatch(line)) {
       section = SSA_SECTION_FONTS;
       add_to_global = false;
 
