@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_wav.h 2900 2005-04-29 08:56:01Z mosu $
+   $Id: r_wav.h 3490 2007-02-06 15:52:01Z mosu $
 
    class definitions for the WAV reader module
 
@@ -35,12 +35,14 @@ class wav_reader_c: public generic_reader_c {
 private:
   unsigned char *chunk;
   mm_io_c *io;
-  int dts_swap_bytes, dts_14_16;
   int bps;
   struct wave_header wheader;
   int64_t bytes_processed;
-  bool is_dts;
+
+  bool is_dts, dts_swap_bytes, dts_14_16;
   dts_header_t dtsheader;
+  unsigned short *buf[2];
+  int cur_buf;
 
 public:
   wav_reader_c(track_info_c &_ti) throw (error_c);
@@ -54,6 +56,9 @@ public:
   virtual int get_progress();
 
   static int probe_file(mm_io_c *io, int64_t size);
+
+protected:
+  virtual int decode_buffer(int len);
 };
 
 #endif // __R_WAV_H
