@@ -8,7 +8,7 @@
    or visit http://www.gnu.org/copyleft/gpl.html
 
    \file
-   \version $Id: mkvmerge.cpp 3341 2006-11-07 15:11:49Z mosu $
+   \version $Id: mkvmerge.cpp 3485 2007-01-30 20:02:53Z mosu $
 
    \author Written by Moritz Bunkus <moritz@bunkus.org>.
    \author Modified by Steve Lhomme <steve.lhomme@free.fr>.
@@ -18,6 +18,7 @@
 
 #include <errno.h>
 #include <ctype.h>
+#include <pcrecpp.h>
 #if defined(SYS_UNIX) || defined(COMP_CYGWIN) || defined(SYS_APPLE)
 #include <signal.h>
 #endif
@@ -75,6 +76,7 @@ file_type_t file_types[] =
    {"ac3 ", FILE_TYPE_AC3, "A/52 (aka AC3)"},
    {"avi ", FILE_TYPE_AVI, "AVI (Audio/Video Interleaved)"},
    {"dts ", FILE_TYPE_DTS, "DTS (Digital Theater System)"},
+   {"h264", FILE_TYPE_AVC_ES, "AVC/h.264 elementary streams"},
    {"mp2 ", FILE_TYPE_MP3, "MPEG-1 layer II audio (CBR and VBR/ABR)"},
    {"mp3 ", FILE_TYPE_MP3, "MPEG-1 layer III audio (CBR and VBR/ABR)"},
    {"mkv ", FILE_TYPE_MATROSKA, "general Matroska files"},
@@ -102,10 +104,15 @@ file_type_t file_types[] =
    {"    ", -1,      "FLAC"},
 #endif
    {"    ", -1,      "MP3 audio"},
+   {"    ", -1,      "RealAudio and RealVideo"},
    {"    ", -1,      "simple text subtitles"},
+   {"    ", -1,      "text subtitles (SRT, SSA/ASS)"},
+   {"    ", -1,      "TTA lossless audio"},
    {"    ", -1,      "uncompressed PCM audio"},
-   {"    ", -1,      "Video (not MPEG1/2)"},
+   {"    ", -1,      "video (MPEG 1, MPEG 2, MPEG 4 part 2 and 10)"},
+   {"    ", -1,      "VobSub subtitles"},
    {"    ", -1,      "Vorbis audio"},
+   {"    ", -1,      "WAVPACK lossless audio"},
    {NULL,  -1,      NULL}};
 
 /** \brief Outputs usage information
@@ -202,7 +209,9 @@ set_usage() {
     "  --delay <TID:Xs|ms|us|ns>\n"
     "                           Delay to apply to the packets of the track\n"
     "                           by simply adjusting the timecodes.\n"
-    "  --default-track <TID>    Sets the 'default' flag for this track.\n"
+    "  --default-track <TID[:bool]>\n"
+    "                           Sets the 'default' flag for this track or\n"
+    "                           forces it not to be present if bool is 0.\n"
     "  --blockadd <TID:x>       Sets the max number of block additional\n"
     "                           levels for this track.\n"
     "  --track-name <TID:name>  Sets the name for a track.\n"
@@ -214,8 +223,13 @@ set_usage() {
     "  -t, --tags <TID:file>    Read tags for the track from a XML file.\n"
     "  --aac-is-sbr <TID>       Track with the ID is HE-AAC/AAC+/SBR-AAC.\n"
     "  --timecodes <TID:file>   Read the timecodes to be used from a file.\n"
-    "  --default-duration <TID:Xms|us|ns>\n"
+    "  --default-duration <TID:Xs|ms|us|ns|fps>\n"
     "                           Force the default duration of a track to X.\n"
+    "                           X can be a floating point number or a fration."
+    "\n"
+    "  --nalu-size-length <TID:n>\n"
+    "                           Force the NALU size length to n bytes with\n"
+    "                           2 <= n <= 4 with 4 being the default.\n"
     "\n Options that only apply to video tracks:\n"
     "  -f, --fourcc <FOURCC>    Forces the FourCC to the specified value.\n"
     "                           Works only for video tracks.\n"
@@ -370,51 +384,85 @@ identify(const string &filename) {
 
 /** \brief Parse a number postfixed with a time-based unit
 
-   This function parsers a number that is postfixed with one of the four
-   units 's', 'ms', 'us' or 'ns'. It returns a number of nanoseconds.
+   This function parsers a number that is postfixed with one of the
+   four units 's', 'ms', 'us' or 'ns'. If the postfix is 'fps' then
+   this means 'frames per second'. It returns a number of nanoseconds.
 */
 int64_t
 parse_number_with_unit(const string &s,
                        const string &subject,
                        const string &argument,
                        string display_s = "") {
-  string unit;
-  int64_t value, multiplier, unit_length;
+  pcrecpp::RE_Options opt_caseless;
+  opt_caseless.set_caseless(true);
+  pcrecpp::RE re1("(-?\\d+\\.?\\d*)(s|ms|us|ns|fps)?", opt_caseless);
+  pcrecpp::RE re2("(-?\\d+)/(-?\\d+)(s|ms|us|ns|fps)?", opt_caseless);
+  string unit, s_n, s_d;
+  int64_t n, d, multiplier;
+  double d_value = 0.0;
+  bool is_fraction = false;
 
   if (display_s == "")
     display_s = s;
 
-  if (s.length() < 2)
+  if (re1.FullMatch(s, &s_n, &unit)) {
+    parse_double(s_n, d_value);
+    d = 1;
+
+  } else if (re2.FullMatch(s, &s_n, &s_d, &unit)) {
+    parse_int(s_n, n);
+    parse_int(s_d, d);
+    is_fraction = true;
+
+  } else
     mxerror(_("'%s' is not a valid %s in '%s %s'.\n"),
             s.c_str(), subject.c_str(), argument.c_str(), display_s.c_str());
 
-  unit = s.substr(s.length() - 2, 2);
-
   multiplier = 1000000000;
-  unit_length = 2;
+
+  unit = downcase(unit);
+
   if (unit == "ms")
     multiplier = 1000000;
   else if (unit == "us")
     multiplier = 1000;
   else if (unit == "ns")
     multiplier = 1;
-  else if (unit.substr(1, 1) == "s")
-    unit_length = 1;
-  else
+  else if (unit == "fps") {
+    if (is_fraction)
+      return 1000000000ll * d / n;
+    else if (29.97 == d_value)
+      return (int64_t)(100100000.0 / 3.0);
+    else if (23.976 == d_value)
+      return (int64_t)(1001000000.0 / 24.0);
+    else
+      return (int64_t)(1000000000.0 / d_value);
+  } else if (unit != "s")
     mxerror(_("'%s' does not contain a valid unit ('s', 'ms', 'us' or 'ns') "
               "in '%s %s'.\n"), s.c_str(), argument.c_str(),
             display_s.c_str());
 
-  if (s.length() < (unit_length + 1))
-    mxerror(_("'%s' is not a valid %s in '%s %s'.\n"),
-            s.c_str(), subject.c_str(), argument.c_str(), display_s.c_str());
+  if (is_fraction)
+    return multiplier * n / d;
+  else
+    return (int64_t)(multiplier * d_value);
+}
 
-  if (!parse_int(s.substr(0, s.length() - unit_length), value))
-    mxerror(_("'%s' does not contain a valid number in front of the unit in "
-              "'%s %s'.\n"), s.c_str(), argument.c_str(), display_s.c_str());
-  value *= multiplier;
+/** \brief Parse a string for a boolean value
 
-  return value;
+   Interpretes the string \c orig as a boolean value. Accepted
+   is "true", "yes", "1" as boolean true and "false", "no", "0"
+   as boolean false.
+*/
+bool
+parse_bool(const string &orig) {
+  string s(downcase(orig));
+
+  if ((s == "yes") || (s == "true") || (s == "1"))
+    return true;
+  if ((s == "no") || (s == "false") || (s == "0"))
+    return false;
+  throw false;
 }
 
 /** \brief Parse tags and add them to the list of all tags
@@ -749,7 +797,7 @@ parse_split_timecodes(const string &arg) {
     s.erase(0, 10);
 
   timecodes = split(s, ",");
-  foreach(timecode, timecodes) {
+  mxforeach(timecode, timecodes) {
     if (!parse_timecode(*timecode, split_after))
       mxerror(_("Invalid time for '--split' in '--split %s'. Additional error "
                 "message: %s.\n"), arg.c_str(), timecode_parser_error.c_str());
@@ -881,6 +929,36 @@ parse_delay(const string &s,
                                                 s);
 }
 
+/** \brief Parse the \c --default-track argument
+
+   The argument must have the form \c TID or \c TID:boolean. The former
+   is equivalent to \c TID:1.
+*/
+static void
+parse_default_track(const string &s,
+                    track_info_c &ti) {
+  vector<string> parts;
+  int64_t id;
+  bool is_default = true;
+
+  // Extract the track number.
+  parts = split(s, ":", 2);
+  strip(parts);
+  id = 0;
+  if (!parse_int(parts[0], id))
+    mxerror(_("Invalid track ID specified in '--default-track %s'.\n"),
+            s.c_str());
+
+  try {
+    if (2 == parts.size())
+      is_default = parse_bool(parts[1]);
+  } catch (...) {
+    mxerror(_("Invalid boolean option specified in '--default-track %s'.\n"),
+            s.c_str());
+  }
+  ti.default_track_flags[id] = is_default;
+}
+
 /** \brief Parse the \c --cues argument
 
    The argument must have the form \c TID:cuestyle, e.g. \c 0:none.
@@ -1157,7 +1235,7 @@ parse_append_to(const string &s,
   append_mapping.clear();
   entries = split(s, ",");
   strip(entries);
-  foreach(entry, entries) {
+  mxforeach(entry, entries) {
     if ((mxsscanf((*entry).c_str(), "" LLD ":" LLD ":" LLD ":" LLD,
                   &mapping.src_file_id, &mapping.src_track_id,
                   &mapping.dst_file_id, &mapping.dst_track_id) != 4) ||
@@ -1171,9 +1249,9 @@ parse_append_to(const string &s,
 
 /** \brief Parse the argument for \c --default-duration
 
-   The argument must be a tupel consisting of a track ID and the default
+   The argument must be a tuple consisting of a track ID and the default
    duration separated by a colon. The duration must be postfixed by 'ms',
-   'us' or 'ns'.
+   'us', 'ns' or 'fps' (see \c parse_number_with_unit).
 */
 static void
 parse_default_duration(const string &s,
@@ -1183,7 +1261,7 @@ parse_default_duration(const string &s,
 
   parts = split(s, ":");
   if (parts.size() != 2)
-    mxerror(_("'%s' is not a valid parts of track ID and default duration in "
+    mxerror(_("'%s' is not a valid tuple of track ID and default duration in "
               "'--default-duration %s'.\n"), s.c_str(), s.c_str());
 
   id = 0;
@@ -1195,6 +1273,42 @@ parse_default_duration(const string &s,
     parse_number_with_unit(parts[1], "default duration", "--default-duration");
 }
 
+/** \brief Parse the argument for \c --nalu-size-length
+
+   The argument must be a tuple consisting of a track ID and the NALU size
+   length, an integer between 2 and 4 inclusively.
+*/
+static void
+parse_nalu_size_length(const string &s,
+                       track_info_c &ti) {
+  static bool nalu_size_length_3_warning_printed = false;
+  vector<string> parts;
+  int64_t id, nalu_size_length;
+
+  parts = split(s, ":");
+  if (parts.size() != 2)
+    mxerror(_("'%s' is not a valid tuple of track ID and NALU size length in "
+              "'--nalu-size-length %s'.\n"), s.c_str(), s.c_str());
+
+  id = 0;
+  if (!parse_int(parts[0], id))
+    mxerror(_("'%s' is not a valid track ID in '--nalu-size-length %s'.\n"),
+            parts[0].c_str(), s.c_str());
+
+  if (!parse_int(parts[1], nalu_size_length) ||
+      (2 > nalu_size_length) || (4 < nalu_size_length))
+    mxerror(_("The NALU size length must be a number between 2 and 4 "
+              "inclusively in '--nalu-size-length %s'.\n"), s.c_str());
+
+  if ((3 == nalu_size_length) && !nalu_size_length_3_warning_printed) {
+    nalu_size_length_3_warning_printed = true;
+    mxwarn("Using a NALU size length of 3 bytes might result in tracks "
+           "that won't be decodable with certain AVC/h.264 codecs.\n");
+  }
+
+  ti.nalu_size_lengths[id] = nalu_size_length;
+}
+
 /** \brief Parse the argument for \c --blockadd
 
    The argument must be a tupel consisting of a track ID and the max number
@@ -1223,6 +1337,31 @@ parse_max_blockadd_id(const string &s,
   ti.max_blockadd_ids[id] = max_blockadd_id;
 }
 
+/** \brief Parse the argument for \c --aac-is-sbr
+
+   The argument can either be just a number (the track ID) or a tupel
+   "trackID:number" where the second number is either "0" or "1". If only
+   a track ID is given then "number" is assumed to be "1".
+*/
+static void
+parse_aac_is_sbr(const string &s,
+                 track_info_c &ti) {
+  vector<string> parts;
+  int64_t id;
+
+  parts = split(s, ":", 2);
+
+  if (!parse_int(parts[0], id) || (id < 0))
+    mxerror(_("Invalid track ID specified in '--aac-is-sbr %s'.\n"),
+            s.c_str());
+
+  if ((parts.size() == 2) && (parts[1] != "0") && (parts[1] != "1"))
+    mxerror(_("Invalid boolean specified in '--aac-is-sbr %s'.\n"),
+            s.c_str());
+
+  ti.all_aac_is_sbr[id] = (1 == parts.size()) || (parts[1] == "1");
+}
+
 /** \brief Sets the priority mkvmerge runs with
 
    Depending on the OS different functions are used. On Unix like systems
@@ -1341,7 +1480,6 @@ parse_args(vector<string> args) {
   vector<string>::const_iterator sit;
   string this_arg, next_arg;
   bool no_next_arg, inputs_found;
-  int64_t id;
   attachment_t attachment;
   mm_io_c *io;
 
@@ -1365,7 +1503,7 @@ parse_args(vector<string> args) {
   }
 
   // First parse options that either just print some infos and then exit.
-  foreach(sit, args) {
+  mxforeach(sit, args) {
     this_arg = *sit;
     if ((sit + 1) == args.end())
       next_arg = "";
@@ -1400,7 +1538,7 @@ parse_args(vector<string> args) {
   mxinfo("%s built on %s %s\n", version_info.c_str(), __DATE__, __TIME__);
 
   // Now parse options that are needed right at the beginning.
-  foreach(sit, args) {
+  mxforeach(sit, args) {
     this_arg = *sit;
     if ((sit + 1) == args.end())
       next_arg = "";
@@ -1431,7 +1569,7 @@ parse_args(vector<string> args) {
     usage(2);
   }
 
-  foreach(sit, args) {
+  mxforeach(sit, args) {
     this_arg = *sit;
     if ((sit + 1) == args.end())
       next_arg = "";
@@ -1610,6 +1748,14 @@ parse_args(vector<string> args) {
       if (no_next_arg)
         mxerror(_("'%s' lacks the file name.\n"), this_arg.c_str());
 
+      try {
+        mm_file_io_c test(next_arg);
+      } catch (...) {
+        mxerror(_("The file '%s' cannot be attached because "
+                  "it does not exist or cannot be read.\n"),
+                next_arg.c_str());
+      }
+
       attachment.name = next_arg;
       if (attachment.mime_type == "")
         attachment.mime_type = guess_mime_type_and_report(next_arg);
@@ -1879,13 +2025,9 @@ parse_args(vector<string> args) {
 
     } else if (this_arg == "--default-track") {
       if (no_next_arg)
-        mxerror(_("'--default-track' lacks the track ID.\n"));
+        mxerror(_("'--default-track' lacks its argument.\n"));
 
-      if (!parse_int(next_arg, id))
-        mxerror(_("Invalid track ID specified in '%s %s'.\n"),
-                this_arg.c_str(), next_arg.c_str());
-
-      ti->default_track_flags.push_back(id);
+      parse_default_track(next_arg, *ti);
       sit++;
 
     } else if (this_arg == "--language") {
@@ -1928,11 +2070,7 @@ parse_args(vector<string> args) {
       if (no_next_arg)
         mxerror(_("'%s' lacks the track ID.\n"), this_arg.c_str());
 
-      if (!parse_int(next_arg, id) || (id < 0))
-        mxerror(_("Invalid track ID specified in '%s %s'.\n"),
-                this_arg.c_str(), next_arg.c_str());
-
-      ti->aac_is_sbr.push_back(id);
+      parse_aac_is_sbr(next_arg, *ti);
       sit++;
 
     } else if (this_arg == "--compression") {
@@ -1988,6 +2126,13 @@ parse_args(vector<string> args) {
       parse_default_duration(next_arg, *ti);
       sit++;
 
+    } else if (this_arg == "--nalu-size-length") {
+      if (no_next_arg)
+        mxerror(_("'--nalu-size-length' lacks its argument.\n"));
+
+      parse_nalu_size_length(next_arg, *ti);
+      sit++;
+
     } else if (this_arg == "--print-malloc-report")
       print_malloc_report = true;
 
@@ -2096,7 +2241,7 @@ main(int argc,
   finish_file(true);
 
   end = time(NULL);
-  mxinfo(_("Muxing took %ld second%s.\n"), end - start,
+  mxinfo(_("Muxing took %ld second%s.\n"), (long)(end - start),
          (end - start) == 1 ? "" : "s");
 
   cleanup();
