@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: jobs.cpp 3138 2005-11-01 22:38:53Z mosu $
+   $Id: jobs.cpp 3447 2007-01-08 16:16:21Z mosu $
 
    job queue management dialog
 
@@ -387,9 +387,7 @@ job_dialog::job_dialog(wxWindow *parent):
   siz_all = new wxBoxSizer(wxVERTICAL);
   siz_all->Add(new wxStaticText(this, -1, wxT("Current and past jobs:")),
                0, wxALIGN_LEFT | wxALL, 10);
-  lv_jobs =
-    new wxListView(this, ID_JOBS_LV_JOBS, wxDefaultPosition, wxSize(400, -1),
-                   wxLC_REPORT | wxSUNKEN_BORDER);
+  lv_jobs = new jobdlg_list_view(this, ID_JOBS_LV_JOBS);
 
   item.m_mask = wxLIST_MASK_TEXT;
   item.m_text = wxT("ID");
@@ -406,23 +404,21 @@ job_dialog::job_dialog(wxWindow *parent):
   item.m_text = wxT("Finished on");
   lv_jobs->InsertColumn(5, item);
 
-  if (jobs.size() == 0) {
-    dummy = lv_jobs->InsertItem(0, wxT("12345"));
-    lv_jobs->SetItem(dummy, 1, wxT("aborted"));
-    lv_jobs->SetItem(dummy, 2, wxT("2004-05-06 07:08:09"));
-    lv_jobs->SetItem(dummy, 3, wxT("2004-05-06 07:08:09"));
-    lv_jobs->SetItem(dummy, 4, wxT("2004-05-06 07:08:09"));
-    lv_jobs->SetItem(dummy, 5, wxT("2004-05-06 07:08:09"));
+  for (i = 0; i < jobs.size(); i++)
+    create_list_item(i);
 
-  } else
-    for (i = 0; i < jobs.size(); i++)
-      create_list_item(i);
+  dummy = lv_jobs->InsertItem(0, wxT("12345"));
+  lv_jobs->SetItem(dummy, 1, wxT("aborted"));
+  lv_jobs->SetItem(dummy, 2, wxT("2004-05-06 07:08:09"));
+  lv_jobs->SetItem(dummy, 3, wxT("2004-05-06 07:08:09"));
+  lv_jobs->SetItem(dummy, 4, wxT("2004-05-06 07:08:09"));
+  lv_jobs->SetItem(dummy, 5, wxT("2004-05-06 07:08:09"));
 
   for (i = 0; i < 6; i++)
     lv_jobs->SetColumnWidth(i, wxLIST_AUTOSIZE);
 
-  if (jobs.size() == 0)
-    lv_jobs->DeleteItem(0);
+  lv_jobs->DeleteItem(0);
+
   siz_line = new wxBoxSizer(wxHORIZONTAL);
   siz_line->Add(lv_jobs, 1, wxGROW | wxRIGHT, 10);
 
@@ -522,9 +518,10 @@ job_dialog::create_list_item(int i) {
 }
 
 void
-job_dialog::enable_buttons(bool enable) {
-  b_up->Enable(enable);
-  b_down->Enable(enable);
+job_dialog::enable_buttons(bool enable,
+                           bool enable_up_down) {
+  b_up->Enable(enable && enable_up_down);
+  b_down->Enable(enable && enable_up_down);
   b_reenable->Enable(enable);
   b_disable->Enable(enable);
   b_delete->Enable(enable);
@@ -732,7 +729,8 @@ job_dialog::on_view_log(wxCommandEvent &evt) {
 
 void
 job_dialog::on_item_selected(wxListEvent &evt) {
-  enable_buttons(lv_jobs->GetSelectedItemCount() > 0);
+  enable_buttons(lv_jobs->GetSelectedItemCount() > 0,
+                 lv_jobs->GetSelectedItemCount() < lv_jobs->GetItemCount());
 }
 
 void
@@ -760,6 +758,33 @@ job_dialog::start_jobs(vector<int> &jobs_to_start) {
   }
 }
 
+void
+job_dialog::on_key_pressed(wxKeyEvent &evt) {
+  if (1 == evt.GetKeyCode()) {  // Ctrl-A
+    int i;
+
+    for (i = 0; lv_jobs->GetItemCount() > i; ++i)
+      lv_jobs->Select(i, true);
+    enable_buttons(lv_jobs->GetSelectedItemCount() > 0, false);
+  } else
+    evt.Skip(true);
+}
+
+jobdlg_list_view::jobdlg_list_view(wxWindow *parent,
+                                   wxWindowID id):
+  wxListView(parent, id, wxDefaultPosition, wxSize(400, -1),
+             wxLC_REPORT | wxSUNKEN_BORDER) {
+}
+
+void
+jobdlg_list_view::on_key_pressed(wxKeyEvent &evt) {
+  if (1 == evt.GetKeyCode())    // Ctrl-A
+    ((job_dialog *)GetParent())->on_key_pressed(evt);
+  else
+    evt.Skip(true);
+}
+
+
 IMPLEMENT_CLASS(job_run_dialog, wxDialog);
 BEGIN_EVENT_TABLE(job_run_dialog, wxDialog)
   EVT_BUTTON(ID_JOBS_B_ABORT, job_run_dialog::on_abort)
@@ -786,3 +811,8 @@ BEGIN_EVENT_TABLE(job_dialog, wxDialog)
   EVT_LIST_ITEM_SELECTED(ID_JOBS_LV_JOBS, job_dialog::on_item_selected)
   EVT_LIST_ITEM_DESELECTED(ID_JOBS_LV_JOBS, job_dialog::on_item_selected)
 END_EVENT_TABLE();
+
+IMPLEMENT_CLASS(jobdlg_list_view, wxListView);
+BEGIN_EVENT_TABLE(jobdlg_list_view, wxListView)
+  EVT_CHAR(jobdlg_list_view::on_key_pressed)
+END_EVENT_TABLE();
