@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: mmg.cpp 3358 2006-11-10 18:23:51Z mosu $
+   $Id: mmg.cpp 3493 2007-02-19 08:52:27Z mosu $
 
    main stuff
 
@@ -40,6 +40,7 @@
 #include "commonebml.h"
 #include "extern_data.h"
 #include "jobs.h"
+#include "mkvmerge.h"
 #include "matroskalogo.xpm"
 #include "mmg.h"
 #include "mmg_dialog.h"
@@ -140,6 +141,9 @@ wxString cli_options[][2] = {
   { wxT("--engage old_aac_codecid"),
     wxT("Use the old AAC codec IDs (e.g. 'A_AAC/MPEG4/SBR') instead of the "
         "new one ('A_AAC').") },
+  { wxT("--engage use_codec_state"),
+    wxT("Allows the use of the CodecState element. This is used for e.g. "
+        "MPEG-1/-2 video tracks for storing the sequence headers.") },
   { wxT("--engage cow"),
     wxT("No help available.") },
   { wxT(""), wxT("") }
@@ -590,11 +594,25 @@ default_track_checked(char type) {
   int i;
 
   for (i = 0; i < tracks.size(); i++)
-    if ((tracks[i]->type == type) && tracks[i]->default_track)
+    if ((tracks[i]->type == type) && (1 == tracks[i]->default_track))
       return i;
   return -1;
 }
 
+void
+set_combobox_selection(wxComboBox *cb,
+                       const wxString wanted) {
+  int i, count;
+
+  cb->SetValue(wanted);
+  count = cb->GetCount();
+  for (i = 0; count > i; ++i)
+    if (cb->GetString(i) == wanted) {
+      cb->SetSelection(i);
+      break;
+    }
+}
+
 void
 wxdie(const wxString &errmsg) {
   wxMessageBox(errmsg, wxT("A serious error has occured"),
@@ -651,7 +669,7 @@ mmg_dialog::mmg_dialog():
                       wxT("Adds the current settings as a new job entry to "
                           "the job queue"));
   muxing_menu->Append(ID_M_MUXING_MANAGE_JOBS,
-                      wxT("&Manage jobs"),
+                      wxT("&Manage jobs\tCtrl-J"),
                       wxT("Brings up the job queue editor"));
   muxing_menu->AppendSeparator();
   muxing_menu->Append(ID_M_MUXING_ADD_CLI_OPTIONS,
@@ -887,9 +905,11 @@ mmg_dialog::on_file_new(wxCommandEvent &evt) {
   tc_output->SetValue(wxT(""));
 
   input_page->load(cfg, MMG_CONFIG_FILE_VERSION_MAX);
+  input_page->on_file_new(evt);
   attachments_page->load(cfg, MMG_CONFIG_FILE_VERSION_MAX);
   global_page->load(cfg, MMG_CONFIG_FILE_VERSION_MAX);
   settings_page->load(cfg, MMG_CONFIG_FILE_VERSION_MAX);
+  notebook->SetSelection(0);
 
   delete cfg;
   wxRemoveFile(tmp_name);
@@ -1194,7 +1214,7 @@ mmg_dialog::on_help(wxCommandEvent &evt) {
     potential_help_paths.push_back(wxGetCwd());
 
     help_path = wxT("");
-    foreach(php, potential_help_paths)
+    mxforeach(php, potential_help_paths)
       if (wxFileExists(*php + wxT("/mkvmerge-gui.hhp"))) {
         help_path = *php;
         break;
@@ -1329,7 +1349,7 @@ mmg_dialog::update_command_line() {
   mmg_file_t *f;
   mmg_attachment_t *a;
   wxString sid, old_cmdline, arg, aids, sids, dids, track_order;
-  wxString append_mapping, format;
+  wxString append_mapping;
 
   old_cmdline = cmdline;
   cmdline = wxT("\"") + mkvmerge_path + wxT("\" -o \"") +
@@ -1348,7 +1368,6 @@ mmg_dialog::update_command_line() {
     clargs.Add(settings_page->cob_priority->GetValue());
   }
 
-  format = wxT(LLD);
   for (fidx = 0; fidx < files.size(); fidx++) {
     f = &files[fidx];
     tracks_selected_here = false;
@@ -1364,9 +1383,7 @@ mmg_dialog::update_command_line() {
         continue;
 
       tracks_selected_here = true;
-      // Avoid compiler warnings about mismatching format and arguments
-      // because mingw does not know about the %I64d syntax.
-      sid.Printf(format, t->id);
+      sid = wxLongLong(t->id).ToString();
 
       if (t->type == wxT('a')) {
         no_audio = false;
@@ -1374,9 +1391,10 @@ mmg_dialog::update_command_line() {
           aids += wxT(",");
         aids += sid;
 
-        if (!t->appending && t->aac_is_sbr) {
+        if (!t->appending && (t->aac_is_sbr || t->aac_is_sbr_detected)) {
           clargs.Add(wxT("--aac-is-sbr"));
-          clargs.Add(sid);
+          clargs.Add(wxString::Format(wxT("%s:%d"), sid.c_str(),
+                                      t->aac_is_sbr ? 1 : 0));
         }
 
       } else if (t->type == wxT('v')) {
@@ -1434,9 +1452,10 @@ mmg_dialog::update_command_line() {
         clargs.Add(sid + wxT(":") + t->track_name);
       }
 
-      if (!t->appending && t->default_track) {
+      if (!t->appending && (0 != t->default_track)) {
         clargs.Add(wxT("--default-track"));
-        clargs.Add(sid);
+        clargs.Add(sid + wxT(":") + (1 == t->default_track ? wxT("yes") :
+                                     wxT("no")));
       }
 
       if (!t->appending && (t->tags.Length() > 0)) {
@@ -1459,6 +1478,17 @@ mmg_dialog::update_command_line() {
         clargs.Add(sid + wxT(":") + t->fourcc);
       }
 
+      if (t->fps.Length() > 0) {
+        clargs.Add(wxT("--default-duration"));
+        clargs.Add(sid + wxT(":") + t->fps + wxT("fps"));
+      }
+
+      if (t->uses_avc_es_packetizer && (4 != t->nalu_size_length)) {
+        clargs.Add(wxT("--nalu-size-length"));
+        clargs.Add(wxString::Format(wxT("%s:%d"), sid.c_str(),
+                                    t->nalu_size_length));
+      }
+
       if (!t->appending && (0 != t->stereo_mode)) {
         clargs.Add(wxT("--stereo-mode"));
         clargs.Add(wxString::Format(wxT("%s:%d"), sid.c_str(),
@@ -1474,6 +1504,18 @@ mmg_dialog::update_command_line() {
         clargs.Add(wxT("--timecodes"));
         clargs.Add(sid + wxT(":") + t->timecodes);
       }
+
+      if (t->user_defined.Length() > 0) {
+        vector<wxString> opts;
+        int i;
+
+        opts = split(t->user_defined, wxString(wxT(" ")));
+        for (i = 0; i < opts.size(); i++) {
+          wxString opt = strip(opts[i]);
+          opt.Replace(wxT("<TID>"), sid, true);
+          clargs.Add(opt);
+        }
+      }
     }
 
     if (aids.length() > 0) {
@@ -1748,13 +1790,21 @@ mmg_dialog::set_title_maybe(const wxString &new_title) {
 
 void
 mmg_dialog::set_output_maybe(const wxString &new_output) {
-  wxString output;
-
   if (settings_page->cb_autoset_output_filename->IsChecked() &&
       (new_output.length() > 0) &&
       (tc_output->GetValue().length() == 0)) {
-    output = new_output.BeforeLast('.');
-    output += wxT(".mkv");
+    wxString output(new_output.BeforeLast('.'));
+    bool has_video = false, has_audio = false;
+    vector<mmg_track_t *>::iterator t;
+
+    mxforeach(t, tracks) {
+      if ('v' == (*t)->type) {
+        has_video = true;
+        break;
+      } else if ('a' == (*t)->type)
+        has_audio = true;
+    }
+    output += has_video ? wxT(".mkv") : has_audio ? wxT(".mka") : wxT(".mks");
     tc_output->SetValue(output);
   }
 }
