@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: mmg.h 3463 2007-01-12 16:02:16Z mosu $
+   $Id: mmg.h 3485 2007-01-30 20:02:53Z mosu $
 
    main stuff - declarations
 
@@ -57,7 +57,9 @@ struct mmg_track_t {
   wxString ctype;
   bool enabled, display_dimensions_selected;
 
-  bool default_track, aac_is_sbr, aac_is_sbr_detected;
+  int default_track;
+  bool aac_is_sbr, aac_is_sbr_detected;
+  bool uses_avc_es_packetizer;
   bool track_name_was_present;
   wxString language, track_name, cues, delay, stretch, sub_charset;
   wxString tags, fourcc, aspect_ratio, compression, timecodes, fps;
@@ -72,10 +74,11 @@ struct mmg_track_t {
   mmg_track_t():
     type(0), id(0), source(0),
     enabled(false), display_dimensions_selected(false),
-    default_track(false), aac_is_sbr(false), aac_is_sbr_detected(false),
+    default_track(0), aac_is_sbr(false), aac_is_sbr_detected(false),
+    uses_avc_es_packetizer(false),
     track_name_was_present(false),
     language(wxT("und")), cues(wxT("default")), sub_charset(wxT("default")),
-    nalu_size_length(2),
+    nalu_size_length(4),
     stereo_mode(0),
     appending(false) {};
 };
