@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: tab_global.cpp 3002 2005-08-03 09:18:15Z mosu $
+   $Id: tab_global.cpp 3389 2006-12-27 11:50:36Z mosu $
 
    "global" tab
 
@@ -470,9 +470,9 @@ tab_global::load(wxConfigBase *cfg,
     }
   }
   cfg->Read(wxT("split_after_bytes"), &s);
-  cob_split_by_size->SetValue(s);
+  set_combobox_selection(cob_split_by_size, s);
   cfg->Read(wxT("split_after_time"), &s);
-  cob_split_by_time->SetValue(s);
+  set_combobox_selection(cob_split_by_time, s);
   cfg->Read(wxT("split_after_timecodes"), &s);
   tc_split_after_timecodes->SetValue(s);
   cfg->Read(wxT("split_max_files"), &s);
@@ -496,9 +496,9 @@ tab_global::load(wxConfigBase *cfg,
   cfg->Read(wxT("chapters"), &s);
   tc_chapters->SetValue(s);
   cfg->Read(wxT("chapter_language"), &s);
-  cob_chap_language->SetValue(s);
+  set_combobox_selection(cob_chap_language, s);
   cfg->Read(wxT("chapter_charset"), &s);
-  cob_chap_charset->SetValue(s);
+  set_combobox_selection(cob_chap_charset, s);
   cfg->Read(wxT("cue_name_format"), &s);
   tc_cue_name_format->SetValue(s);
 
@@ -639,7 +639,7 @@ tab_global::is_valid_split_timecode_list() {
   vector<string>::const_iterator i;
 
   parts = split(s, ",");
-  foreach(i, parts)
+  mxforeach(i, parts)
     if (!is_valid_split_timecode(*i, wxT("timecode list")))
       return false;
 
