@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: tab_input.cpp 3343 2006-11-08 15:20:02Z mosu $
+   $Id: tab_input.cpp 3483 2007-01-30 09:24:59Z mosu $
 
    "input" tab
 
@@ -35,6 +35,7 @@
 #include "mmg_dialog.h"
 #include "tab_input.h"
 #include "tab_global.h"
+#include "tab_settings.h"
 
 using namespace std;
 
@@ -64,31 +65,6 @@ const wxChar *predefined_fourccs[] = {
   NULL
 };
 
-void
-set_combobox_selection(wxComboBox *cb,
-                       const wxChar * const string_list[],
-                       const wxString wanted) {
-  int i;
-  bool found;
-
-  cb->SetValue(wanted);
-  found = false;
-  for (i = 0; NULL != string_list[i]; ++i)
-    if (wanted == string_list[i]) {
-      cb->SetSelection(i);
-      found = true;
-      break;
-    }
-}
-
-wxString
-get_combobox_selection(wxComboBox *cb) {
-  if (wxNOT_FOUND != cb->GetSelection())
-    return cb->GetStringSelection();
-  else
-    return cb->GetValue();
-}
-
 class input_drop_target_c: public wxFileDropTarget {
 private:
   tab_input *owner;
@@ -200,9 +176,11 @@ tab_input::tab_input(wxWindow *parent):
 
   ti_general = new tab_input_general(nb_options, this);
   ti_format = new tab_input_format(nb_options, this);
+  ti_extra = new tab_input_extra(nb_options, this);
 
   nb_options->AddPage(ti_general, wxT("General track options"));
   nb_options->AddPage(ti_format, wxT("Format specific options"));
+  nb_options->AddPage(ti_extra, wxT("Extra options"));
 
   siz_all->Add(nb_options, 0, wxGROW | wxLEFT | wxRIGHT, LEFTRIGHTSPACING);
 
@@ -219,20 +197,26 @@ tab_input::tab_input(wxWindow *parent):
   value_copy_timer.Start(333);
 
   SetDropTarget(new input_drop_target_c(this));
+
+  wxConfig *cfg = (wxConfig *)wxConfigBase::Get();
+
+  cfg->SetPath(wxT("/GUI"));
+  cfg->Read(wxT("avc_es_fps_warning_shown"), &avc_es_fps_warning_shown, false);
 }
 
 void
 tab_input::set_track_mode(mmg_track_t *t) {
-  nb_options->Enable(NULL != t);
   ti_general->set_track_mode(t);
   ti_format->set_track_mode(t);
+  ti_extra->set_track_mode(t);
 }
 
 void
 tab_input::select_file(bool append) {
   static struct { wxChar *title, *extensions; } file_types[] = {
-    { wxT("AAC (Advanced Audio Coding)"), wxT("aac m4a mp4") },
     { wxT("A/52 (aka AC3)"), wxT("ac3") },
+    { wxT("AAC (Advanced Audio Coding)"), wxT("aac m4a mp4") },
+    { wxT("AVC/h.264 elementary streams"), wxT("264 avc h264 x264") },
     { wxT("AVI (Audio/Video Interleaved)"), wxT("avi") },
     { wxT("DTS (Digital Theater System)"), wxT("dts") },
     { wxT("FLAC (Free Lossless Audio Codec)"), wxT("flac ogg") },
@@ -339,6 +323,9 @@ tab_input::add_file(const wxString &file_name,
   bool default_video_track_found, default_audio_track_found;
   bool default_subtitle_track_found;
 
+  wxFileName file_name_obj(file_name);
+  last_open_dir = file_name_obj.GetPath();
+
   opt_file_name.Printf(wxT("%smmg-mkvmerge-options-%d-%d"),
                        get_temp_dir().c_str(),
                        (int)wxGetProcessId(), (int)time(NULL));
@@ -399,9 +386,11 @@ tab_input::add_file(const wxString &file_name,
   }
 
   wxString delay_from_file_name;
-  wxRegEx re_delay(wxT("delay[[:blank:]]+(-?[[:digit:]]+)"), wxRE_ICASE);
-  if (re_delay.Matches(file_name))
-    delay_from_file_name = re_delay.GetMatch(file_name, 1);
+  if (mdlg->settings_page->cb_set_delay_from_filename->IsChecked()) {
+    wxRegEx re_delay(wxT("delay[[:blank:]]+(-?[[:digit:]]+)"), wxRE_ICASE);
+    if (re_delay.Matches(file_name))
+      delay_from_file_name = re_delay.GetMatch(file_name, 1);
+  }
 
   default_audio_track_found = -1 != default_track_checked('a');
   default_video_track_found = -1 != default_track_checked('v');
@@ -435,6 +424,11 @@ tab_input::add_file(const wxString &file_name,
       if (info.length() > 0) {
         args = split(info, wxU(" "));
         for (k = 0; k < args.size(); k++) {
+          if (args[k] == wxT("uses_avc_es_packetizer")) {
+            track->uses_avc_es_packetizer = true;
+            continue;
+          }
+
           pair = split(args[k], wxU(":"), 2);
           if (pair.size() != 2)
             continue;
@@ -479,7 +473,10 @@ tab_input::add_file(const wxString &file_name,
             parse_int(wxMB(pair[1]), track->stereo_mode);
             track->stereo_mode += 1;
 
-          }
+          } else if (pair[0] == wxT("aac_is_sbr"))
+            track->aac_is_sbr = track->aac_is_sbr_detected =
+              pair[1] == wxT("true");
+
         }
       }
 
@@ -490,6 +487,31 @@ tab_input::add_file(const wxString &file_name,
 
       file.tracks.push_back(track);
 
+      if ((FILE_TYPE_AVC_ES == file.container) &&
+          ('v' == track->type) &&
+          (track->ctype.Find(wxT("MPEG-4 part 10 ES")) >= 0) &&
+          (!avc_es_fps_warning_shown ||
+           mdlg->settings_page->cb_warn_usage->GetValue())) {
+        wxMessageBox(wxT("You're adding an AVC/h.264 elementary stream to "
+                         "the output file. "
+                         "mkvmerge cannot determine the number of frames "
+                         "per second for such files itself. "
+                         "Therefore you have to set this parameter "
+                         "yourself on the 'format specific options' page."
+                         "\n\n"
+                         "If you don't do this then mkvmerge will assume "
+                         "25 fps.\n\n"
+                         "This message will only be shown once unless you "
+                         "have enabled mmg's warnings on its 'settings' "
+                         "page."));
+        avc_es_fps_warning_shown = true;
+
+        wxConfig *cfg = (wxConfig *)wxConfigBase::Get();
+        cfg->SetPath(wxT("/GUI"));
+        cfg->Write(wxT("avc_es_fps_warning_shown"), true);
+        cfg->Flush();
+      }
+
     } else if ((pos = output[i].Find(wxT("container:"))) > 0) {
       wxString container, info;
 
@@ -522,6 +544,8 @@ tab_input::add_file(const wxString &file_name,
         file.container = FILE_TYPE_VOBSUB;
       else if (container == wxT("WAV"))
         file.container = FILE_TYPE_WAV;
+      else if (container == wxT("AVC/h.264"))
+        file.container = FILE_TYPE_AVC_ES;
       else
         file.container = FILE_TYPE_IS_UNKNOWN;
 
@@ -571,7 +595,6 @@ tab_input::add_file(const wxString &file_name,
   mdlg->set_title_maybe(file.title);
   if ((file.container == FILE_TYPE_OGM) && (video_track_name.Length() > 0))
     mdlg->set_title_maybe(video_track_name);
-  mdlg->set_output_maybe(file.file_name);
   file.appending = append;
   files.insert(files.begin() + new_file_pos, file);
 
@@ -637,12 +660,11 @@ tab_input::add_file(const wxString &file_name,
     clb_tracks->Check(new_track_pos, true);
   }
 
+  mdlg->set_output_maybe(file.file_name);
+
   st_tracks->Enable(true);
   clb_tracks->Enable(true);
   b_append_file->Enable(true);
-
-  wxFileName file_name_obj(file_name);
-  last_open_dir = file_name_obj.GetPath();
 }
 
 void
@@ -697,20 +719,21 @@ tab_input::on_remove_file(wxCommandEvent &evt) {
 
   files.erase(files.begin() + selected_file);
   lb_input_files->Delete(selected_file);
-  selected_file = -1;
-  st_file_options->Enable(false);
-  cb_no_chapters->Enable(false);
-  cb_no_attachments->Enable(false);
-  cb_no_tags->Enable(false);
-  b_remove_file->Enable(false);
+  selected_file = lb_input_files->GetSelection();
+  st_file_options->Enable(-1 != selected_file);
+  cb_no_chapters->Enable(-1 != selected_file);
+  cb_no_attachments->Enable(-1 != selected_file);
+  cb_no_tags->Enable(-1 != selected_file);
+  b_remove_file->Enable(-1 != selected_file);
   b_append_file->Enable(tracks.size() > 0);
-  b_track_up->Enable(false);
-  b_track_down->Enable(false);
-  set_track_mode(NULL);
+  b_track_up->Enable(-1 != selected_file);
+  b_track_down->Enable(-1 != selected_file);
+
+  selected_track = clb_tracks->GetSelection();
+  set_track_mode(-1 == selected_track ? NULL : tracks[selected_track]);
   if (tracks.size() == 0) {
     st_tracks->Enable(false);
     clb_tracks->Enable(false);
-    selected_track = -1;
     mdlg->remove_output_filename();
   }
 
@@ -819,19 +842,19 @@ tab_input::on_file_selected(wxCommandEvent &evt) {
 
 void
 tab_input::on_nochapters_clicked(wxCommandEvent &evt) {
-  if (selected_file != -1)
+  if (dont_copy_values_now || (selected_file != -1))
     files[selected_file].no_chapters = cb_no_chapters->GetValue();
 }
 
 void
 tab_input::on_noattachments_clicked(wxCommandEvent &evt) {
-  if (selected_file != -1)
+  if (dont_copy_values_now || (selected_file != -1))
     files[selected_file].no_attachments = cb_no_attachments->GetValue();
 }
 
 void
 tab_input::on_notags_clicked(wxCommandEvent &evt) {
-  if (selected_file != -1)
+  if (dont_copy_values_now || (selected_file != -1))
     files[selected_file].no_tags = cb_no_tags->GetValue();
 }
 
@@ -871,18 +894,20 @@ tab_input::on_track_selected(wxCommandEvent &evt) {
   ti_format->tc_stretch->SetValue(t->stretch);
   ti_format->cob_sub_charset->SetValue(t->sub_charset);
   ti_general->tc_tags->SetValue(t->tags);
-  ti_general->cb_default->SetValue(t->default_track);
+  ti_general->cob_default->SetSelection(t->default_track);
   ti_format->cb_aac_is_sbr->SetValue(t->aac_is_sbr);
-  set_combobox_selection(ti_format->cob_aspect_ratio, predefined_aspect_ratios,
-                         t->aspect_ratio);
+  set_combobox_selection(ti_format->cob_aspect_ratio, t->aspect_ratio);
   ti_format->tc_display_width->SetValue(t->dwidth);
   ti_format->tc_display_height->SetValue(t->dheight);
   selected_track = new_sel;
   ti_format->cob_compression->SetValue(t->compression);
   ti_general->tc_timecodes->SetValue(t->timecodes);
-  set_combobox_selection(ti_format->cob_fourcc, predefined_fourccs, t->fourcc);
+  set_combobox_selection(ti_format->cob_fourcc, t->fourcc);
+  set_combobox_selection(ti_format->cob_fps, t->fps);
+  ti_format->cob_nalu_size_length->SetSelection(t->nalu_size_length - 2);
   ti_general->tc_track_name->SetFocus();
   ti_format->cob_stereo_mode->SetSelection(t->stereo_mode);
+  ti_extra->tc_user_defined->SetValue(t->user_defined);
 
   dont_copy_values_now = false;
 }
@@ -903,8 +928,24 @@ tab_input::on_value_copy_timer(wxTimerEvent &evt) {
     return;
 
   t = tracks[selected_track];
-  t->aspect_ratio = get_combobox_selection(ti_format->cob_aspect_ratio);
-  t->fourcc = get_combobox_selection(ti_format->cob_fourcc);
+  t->aspect_ratio = ti_format->cob_aspect_ratio->GetValue();
+  t->fourcc = ti_format->cob_fourcc->GetValue();
+  t->fps = ti_format->cob_fps->GetValue();
+}
+
+void
+tab_input::on_file_new(wxCommandEvent &evt) {
+  b_append_file->Enable(false);
+  b_remove_file->Enable(false);
+  st_file_options->Enable(false);
+  cb_no_chapters->Enable(false);
+  cb_no_attachments->Enable(false);
+  cb_no_tags->Enable(false);
+  cb_no_chapters->SetValue(false);
+  cb_no_attachments->SetValue(false);
+  cb_no_tags->SetValue(false);
+  b_track_up->Enable(false);
+  b_track_down->Enable(false);
 }
 
 void
@@ -940,7 +981,7 @@ tab_input::save(wxConfigBase *cfg) {
       cfg->Write(wxT("id"), s);
       cfg->Write(wxT("enabled"), t->enabled);
       cfg->Write(wxT("content_type"), t->ctype);
-      cfg->Write(wxT("default_track"), t->default_track);
+      cfg->Write(wxT("default_track_2"), t->default_track);
       cfg->Write(wxT("aac_is_sbr"), t->aac_is_sbr);
       cfg->Write(wxT("language"), t->language);
       cfg->Write(wxT("track_name"), t->track_name);
@@ -956,10 +997,13 @@ tab_input::save(wxConfigBase *cfg) {
       cfg->Write(wxT("display_width"), t->dwidth);
       cfg->Write(wxT("display_height"), t->dheight);
       cfg->Write(wxT("fourcc"), t->fourcc);
+      cfg->Write(wxT("fps"), t->fps);
+      cfg->Write(wxT("nalu_size_length"), t->nalu_size_length);
       cfg->Write(wxT("stereo_mode"), t->stereo_mode);
       cfg->Write(wxT("compression"), t->compression);
       cfg->Write(wxT("track_name_was_present"), t->track_name_was_present);
       cfg->Write(wxT("appending"), t->appending);
+      cfg->Write(wxT("user_defined"), t->user_defined);
 
       cfg->SetPath(wxT(".."));
     }
@@ -1020,6 +1064,7 @@ tab_input::load(wxConfigBase *cfg,
 
     for (tidx = 0; tidx < (uint32_t)num_tracks; tidx++) {
       mmg_track_ptr tr(new mmg_track_t);
+      bool dummy = false;
 
       s.Printf(wxT("track %ld"), tidx);
       cfg->SetPath(s);
@@ -1036,7 +1081,10 @@ tab_input::load(wxConfigBase *cfg,
       }
       cfg->Read(wxT("enabled"), &tr->enabled);
       cfg->Read(wxT("content_type"), &tr->ctype);
-      cfg->Read(wxT("default_track"), &tr->default_track, false);
+      if (cfg->Read(wxT("default_track"), &dummy)) {
+        tr->default_track = dummy ? 1 : 0;
+      } else
+        cfg->Read(wxT("default_track_2"), &tr->default_track, 0);
       cfg->Read(wxT("aac_is_sbr"), &tr->aac_is_sbr, false);
       cfg->Read(wxT("language"), &tr->language);
       cfg->Read(wxT("track_name"), &tr->track_name);
@@ -1051,12 +1099,15 @@ tab_input::load(wxConfigBase *cfg,
       cfg->Read(wxT("display_width"), &tr->dwidth);
       cfg->Read(wxT("display_height"), &tr->dheight);
       cfg->Read(wxT("fourcc"), &tr->fourcc);
+      cfg->Read(wxT("fps"), &tr->fps);
+      cfg->Read(wxT("nalu_size_length"), &tr->nalu_size_length, 2);
       cfg->Read(wxT("stereo_mode"), &tr->stereo_mode, 0);
       cfg->Read(wxT("compression"), &tr->compression);
       cfg->Read(wxT("timecodes"), &tr->timecodes);
       cfg->Read(wxT("track_name_was_present"), &tr->track_name_was_present,
                 false);
       cfg->Read(wxT("appending"), &tr->appending, false);
+      cfg->Read(wxT("user_defined"), &tr->user_defined);
       tr->source = files.size();
       if (track_order.Length() > 0)
         track_order += wxT(",");
@@ -1147,7 +1198,7 @@ tab_input::validate_settings() {
   bool tracks_selected, dot_present, ok;
   int64_t dummy_i;
   string format, s;
-  wxString sid;
+  wxString sid, str;
 
   // Appending a file to itself is not allowed.
   for (tidx = 2; tidx < tracks.size(); tidx++)
@@ -1187,13 +1238,13 @@ tab_input::validate_settings() {
         return false;
       }
 
-      sid = t->stretch;
-      strip(sid);
-      if (sid.length() > 0) {
+      str = t->stretch;
+      strip(str);
+      if (str.length() > 0) {
         wxRegEx re_stretch(wxT("^[[:digit:]]+([,\\.][[:digit:]]+)?"
                                "(/[[:digit:]]+([,\\.][[:digit:]]+)?)?$"),
                            wxRE_ICASE);
-        if (!re_stretch.Matches(sid)) {
+        if (!re_stretch.Matches(str)) {
           wxMessageBox(wxT("The stretch setting for track nr. ") + sid +
                        wxT(" in file '") + f->file_name +
                        wxT("' is invalid."), wxT("mkvmerge GUI: error"),
@@ -1213,6 +1264,40 @@ tab_input::validate_settings() {
         return false;
       }
 
+      str = t->fps;
+      strip(str);
+      if (str.length() > 0) {
+        wxRegEx re_fps1(wxT("^[[:digit:]]+\\.?[[:digit:]]*$"));
+        wxRegEx re_fps2(wxT("^[[:digit:]]+/[[:digit:]]+$"));
+        if (!re_fps1.Matches(str) && !re_fps2.Matches(str)) {
+          wxMessageBox(wxT("The FPS setting for track nr. ") + sid +
+                       wxT(" in file '") + f->file_name +
+                       wxT("' is invalid."), wxT("mkvmerge GUI: error"),
+                       wxOK | wxCENTER | wxICON_ERROR);
+          return false;
+        }
+      } else if ((FILE_TYPE_AVC_ES == f->container) &&
+                 ('v' == t->type) &&
+                 (t->ctype.Find(wxT("MPEG-4 part 10 ES")) >= 0) &&
+                 mdlg->settings_page->cb_warn_usage->GetValue()) {
+        wxString message =
+          wxString::Format(wxT("You haven't selected a number of frames "
+                               "per second for track " LLD " of file '%s'. "
+                               "mkvmerge cannot determine the number of "
+                               "frames per second for such files itself. "
+                               "Therefore you have to set this parameter "
+                               "yourself on the 'format specific options' "
+                               "page."
+                               "\n\n"
+                               "If you don't do this then mkvmerge will "
+                               "assume 25 fps.\n\n"
+                               "Do you still want to continue?"),
+                           t->id, f->file_name.c_str());
+        if (wxMessageBox(message, wxT("No FPS selected for AVC/h.264 track"),
+                         wxYES | wxNO, mdlg) == wxNO)
+          return false;
+      }
+
       s = wxMB(t->aspect_ratio);
       strip(s);
       if (s.length() > 0) {
