@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: tab_input.h 3343 2006-11-08 15:20:02Z mosu $
+   $Id: tab_input.h 3480 2007-01-28 21:57:32Z mosu $
 
    declarations for the input tab
 
@@ -51,6 +51,9 @@
 #define ID_T_INPUTVALUES                  11030
 #define ID_B_APPENDFILE                   11031
 #define ID_CB_STEREO_MODE                 11032
+#define ID_CB_FPS                         11033
+#define ID_CB_NALU_SIZE_LENGTH            11034
+#define ID_TC_USER_DEFINED                11035
 #define ID_NB_OPTIONS                     11999
 
 extern const wxChar *predefined_aspect_ratios[];
@@ -62,12 +65,11 @@ class tab_input_general: public wxPanel {
   DECLARE_CLASS(tab_input_general);
   DECLARE_EVENT_TABLE();
 public:
-  wxCheckBox *cb_default;
-  wxComboBox *cob_language, *cob_cues;
+  wxComboBox *cob_default, *cob_language, *cob_cues;
   wxTextCtrl *tc_track_name, *tc_tags, *tc_timecodes;
   wxButton *b_browse_tags, *b_browse_timecodes;
 
-  wxStaticText *st_language, *st_track_name;
+  wxStaticText *st_language, *st_track_name, *st_default;
   wxStaticText *st_cues, *st_tags, *st_timecodes;
 
   tab_input *input;
@@ -77,9 +79,10 @@ public:
 
   void setup_languages();
   void setup_cues();
+  void setup_default_track();
   void set_track_mode(mmg_track_t *t);
 
-  void on_default_track_clicked(wxCommandEvent &evt);
+  void on_default_track_selected(wxCommandEvent &evt);
   void on_language_selected(wxCommandEvent &evt);
   void on_cues_selected(wxCommandEvent &evt);
   void on_browse_tags(wxCommandEvent &evt);
@@ -95,13 +98,14 @@ class tab_input_format: public wxPanel {
 public:
   wxCheckBox *cb_aac_is_sbr;
   wxComboBox *cob_sub_charset, *cob_aspect_ratio, *cob_fourcc, *cob_compression;
-  wxComboBox *cob_stereo_mode;
+  wxComboBox *cob_stereo_mode, *cob_fps, *cob_nalu_size_length;
   wxTextCtrl *tc_delay, *tc_stretch;
   wxRadioButton *rb_aspect_ratio, *rb_display_dimensions;
   wxTextCtrl *tc_display_width, *tc_display_height;
 
   wxStaticText *st_delay, *st_stretch, *st_stereo_mode;
   wxStaticText *st_x, *st_sub_charset, *st_fourcc, *st_compression;
+  wxStaticText *st_fps, *st_nalu_size_length;
 
   tab_input *input;
 
@@ -121,16 +125,37 @@ public:
   void on_display_width_changed(wxCommandEvent &evt);
   void on_display_height_changed(wxCommandEvent &evt);
   void on_fourcc_changed(wxCommandEvent &evt);
+  void on_fps_changed(wxCommandEvent &evt);
+  void on_nalu_size_length_changed(wxCommandEvent &evt);
   void on_stereo_mode_changed(wxCommandEvent &evt);
   void on_compression_selected(wxCommandEvent &evt);
 };
 
+class tab_input_extra: public wxPanel {
+  DECLARE_CLASS(tab_input_extra);
+  DECLARE_EVENT_TABLE();
+public:
+  wxTextCtrl *tc_user_defined;
+
+  wxStaticText *st_user_defined;
+
+  tab_input *input;
+
+public:
+  tab_input_extra(wxWindow *parent, tab_input *ti);
+
+  void set_track_mode(mmg_track_t *t);
+
+  void on_user_defined_changed(wxCommandEvent &evt);
+};
+
 class tab_input: public wxPanel {
   DECLARE_CLASS(tab_input);
   DECLARE_EVENT_TABLE();
 public:
   tab_input_general *ti_general;
   tab_input_format *ti_format;
+  tab_input_extra *ti_extra;
 
   wxListBox *lb_input_files;
   wxButton *b_add_file, *b_remove_file;
@@ -142,6 +167,7 @@ public:
 
   wxTimer value_copy_timer;
   bool dont_copy_values_now;
+  bool avc_es_fps_warning_shown;
 
   int selected_file, selected_track;
 
@@ -162,6 +188,7 @@ public:
   void on_noattachments_clicked(wxCommandEvent &evt);
   void on_notags_clicked(wxCommandEvent &evt);
   void on_value_copy_timer(wxTimerEvent &evt);
+  void on_file_new(wxCommandEvent &evt);
 
   void set_track_mode(mmg_track_t *t);
 
