@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: tab_input_format.cpp 3343 2006-11-08 15:20:02Z mosu $
+   $Id: tab_input_format.cpp 3485 2007-01-30 20:02:53Z mosu $
 
    "input" tab -- "Format specific track options" sub-tab
 
@@ -71,6 +71,7 @@ tab_input_format::tab_input_format(wxWindow *parent,
                                    "which case both numbers must be integer "
                                    "(e.g. 16/9) or just a single floting "
                                    "point number 'f' (e.g. 2.35)."));
+  cob_aspect_ratio->SetSizeHints(0, -1);
   siz_fg->Add(cob_aspect_ratio, 1, wxGROW | wxALIGN_CENTER_VERTICAL | wxALL,
               STDSPACING);
 
@@ -86,6 +87,7 @@ tab_input_format::tab_input_format(wxWindow *parent,
   tc_display_width->SetToolTip(TIP("Sets the display width of the track."
                                    "The height must be set as well, or this "
                                    "field will be ignored."));
+  tc_display_width->SetSizeHints(0, -1);
   siz_line->Add(tc_display_width, 1, wxGROW | wxALL, STDSPACING);
 
   st_x = new wxStaticText(this, wxID_STATIC, wxT("x"));
@@ -96,6 +98,7 @@ tab_input_format::tab_input_format(wxWindow *parent,
   tc_display_height->SetToolTip(TIP("Sets the display height of the track."
                                     "The width must be set as well, or this "
                                     "field will be ignored."));
+  tc_display_height->SetSizeHints(0, -1);
   siz_line->Add(tc_display_height, 1, wxGROW | wxALL, STDSPACING);
   siz_fg->Add(siz_line, 1, wxGROW, 0);
 
@@ -131,6 +134,47 @@ tab_input_format::tab_input_format(wxWindow *parent,
   siz_fg->Add(cob_stereo_mode, 1, wxGROW | wxALIGN_CENTER_VERTICAL | wxALL,
               STDSPACING);
 
+  st_fps = new wxStaticText(this, -1, wxT("FPS:"));
+  st_fps->Enable(false);
+  siz_fg->Add(st_fps, 0, wxALIGN_CENTER_VERTICAL | wxALL, STDSPACING);
+
+  cob_fps =
+    new wxComboBox(this, ID_CB_FPS, wxT(""), wxDefaultPosition,
+                   wxDefaultSize, 0, NULL, wxCB_DROPDOWN);
+  cob_fps->SetToolTip(TIP("Sets the default duration or number of frames "
+                          "per second for a track. This is only possible "
+                          "for input formats from which mkvmerge cannot "
+                          "get this information itself. At the moment this "
+                          "only includes AVC/h.264 elementary streams. "
+                          "This can either be a floating point number or "
+                          "a fraction."));
+  cob_fps->SetSizeHints(0, -1);
+  siz_fg->Add(cob_fps, 1, wxGROW | wxALIGN_CENTER_VERTICAL | wxALL,
+              STDSPACING);
+
+  st_nalu_size_length = new wxStaticText(this, -1, wxT("NALU size length:"));
+  st_nalu_size_length->Enable(false);
+  siz_fg->Add(st_nalu_size_length, 0, wxALIGN_CENTER_VERTICAL | wxALL,
+              STDSPACING);
+
+  cob_nalu_size_length =
+    new wxComboBox(this, ID_CB_NALU_SIZE_LENGTH, wxT(""), wxDefaultPosition,
+                   wxDefaultSize, 0, NULL, wxCB_READONLY);
+  cob_nalu_size_length->
+    SetToolTip(TIP("Forces the NALU size length to a certain number of bytes. "
+                   "This parameter is only available for AVC/h.264 elementary "
+                   "streams read from AVC/h.264 ES files, AVIs or Matroska "
+                   "files created with '--engage allow_avc_in_vwf_mode'. "
+                   "It defaults to 4 bytes, but there are files which do not "
+                   "contain a frame or slice that is bigger than 65535 bytes. "
+                   "For such files you can use this parameter and decrease "
+                   "the size to 2. You can also try 3 which results in "
+                   "smaller files but might not be decodable by all AVC/h.264 "
+                   "codecs."));
+  cob_nalu_size_length->SetSizeHints(0, -1);
+  siz_fg->Add(cob_nalu_size_length, 1, wxGROW | wxALIGN_CENTER_VERTICAL | wxALL,
+              STDSPACING);
+
   st_delay = new wxStaticText(this, wxID_STATIC, wxT("Delay (in ms):"));
   st_delay->Enable(false);
   siz_fg->Add(st_delay, 0, wxALIGN_CENTER_VERTICAL | wxALL, STDSPACING);
@@ -227,6 +271,17 @@ tab_input_format::setup_control_contents() {
   cob_stereo_mode->Append(wxT("Right eye"));
   cob_stereo_mode->Append(wxT("Both eyes"));
 
+  cob_fps->Append(wxT(""));
+  cob_fps->Append(wxT("24"));
+  cob_fps->Append(wxT("25"));
+  cob_fps->Append(wxT("30"));
+  cob_fps->Append(wxT("30000/1001"));
+  cob_fps->Append(wxT("24000/1001"));
+
+  cob_nalu_size_length->Append(wxT("2"));
+  cob_nalu_size_length->Append(wxT("3"));
+  cob_nalu_size_length->Append(wxT("4"));
+
   if (sorted_charsets.Count() == 0) {
     for (i = 0; sub_charsets[i] != NULL; i++)
       sorted_charsets.Add(wxU(sub_charsets[i]));
@@ -251,21 +306,26 @@ tab_input_format::set_track_mode(mmg_track_t *t) {
   wxString ctype = t ? t->ctype : wxT("");
   bool appending = t ? t->appending : false;
   bool video = ('v' == type) && !appending;
-  bool audio = ('a' == type) && !appending;
-  bool subs = ('s' == type) && !appending;
+  bool audio_app = ('a' == type);
+  bool subs_app = ('s' == type);
+  bool avc_es = video && t->uses_avc_es_packetizer;
 
   ctype = ctype.Lower();
 
-  st_delay->Enable(audio || subs);
-  tc_delay->Enable(audio || subs);
-  st_stretch->Enable(audio || subs);
-  tc_stretch->Enable(audio || subs);
-  st_sub_charset->Enable(subs && (ctype.Find(wxT("vobsub")) < 0));
-  cob_sub_charset->Enable(subs && (ctype.Find(wxT("vobsub")) < 0));
+  st_delay->Enable(audio_app || subs_app);
+  tc_delay->Enable(audio_app || subs_app);
+  st_stretch->Enable(audio_app || subs_app);
+  tc_stretch->Enable(audio_app || subs_app);
+  st_sub_charset->Enable(subs_app && (ctype.Find(wxT("vobsub")) < 0));
+  cob_sub_charset->Enable(subs_app && (ctype.Find(wxT("vobsub")) < 0));
   st_fourcc->Enable(video);
   cob_fourcc->Enable(video);
   st_stereo_mode->Enable(video);
   cob_stereo_mode->Enable(video);
+  st_fps->Enable(avc_es);
+  cob_fps->Enable(avc_es);
+  st_nalu_size_length->Enable(avc_es);
+  cob_nalu_size_length->Enable(avc_es);
   st_compression->Enable((ctype.Find(wxT("vobsub")) >= 0) && !appending);
   cob_compression->Enable((ctype.Find(wxT("vobsub")) >= 0) && !appending);
 
@@ -280,14 +340,34 @@ tab_input_format::set_track_mode(mmg_track_t *t) {
   rb_aspect_ratio->SetValue(video && ar_enabled);
   rb_display_dimensions->SetValue(video && !ar_enabled);
 
-  cb_aac_is_sbr->Enable(audio &&
+  cb_aac_is_sbr->Enable(audio_app &&
                         ((ctype.Find(wxT("aac")) >= 0) ||
                          (ctype.Find(wxT("mp4a")) >= 0)));
+
+  if (NULL == t) {
+    bool saved_dcvn = input->dont_copy_values_now;
+    input->dont_copy_values_now = true;
+
+    set_combobox_selection(cob_aspect_ratio, wxT(""));
+    tc_display_width->SetValue(wxT(""));
+    tc_display_height->SetValue(wxT(""));
+    set_combobox_selection(cob_fourcc, wxT(""));
+    set_combobox_selection(cob_fps, wxT(""));
+    cob_nalu_size_length->SetSelection(2);
+    set_combobox_selection(cob_stereo_mode, wxT(""));
+    tc_delay->SetValue(wxT(""));
+    tc_stretch->SetValue(wxT(""));
+    set_combobox_selection(cob_sub_charset, wxU(""));
+    set_combobox_selection(cob_compression, wxT(""));
+    cb_aac_is_sbr->SetValue(false);
+
+    input->dont_copy_values_now = saved_dcvn;
+  }
 }
 
 void
 tab_input_format::on_aac_is_sbr_clicked(wxCommandEvent &evt) {
-  if (input->selected_track == -1)
+  if (input->dont_copy_values_now || (input->selected_track == -1))
     return;
 
   tracks[input->selected_track]->aac_is_sbr = cb_aac_is_sbr->GetValue();
@@ -295,7 +375,7 @@ tab_input_format::on_aac_is_sbr_clicked(wxCommandEvent &evt) {
 
 void
 tab_input_format::on_subcharset_selected(wxCommandEvent &evt) {
-  if (input->selected_track == -1)
+  if (input->dont_copy_values_now || (input->selected_track == -1))
     return;
 
   tracks[input->selected_track]->sub_charset =
@@ -304,7 +384,7 @@ tab_input_format::on_subcharset_selected(wxCommandEvent &evt) {
 
 void
 tab_input_format::on_delay_changed(wxCommandEvent &evt) {
-  if (input->selected_track == -1)
+  if (input->dont_copy_values_now || (input->selected_track == -1))
     return;
 
   tracks[input->selected_track]->delay = tc_delay->GetValue();
@@ -312,7 +392,7 @@ tab_input_format::on_delay_changed(wxCommandEvent &evt) {
 
 void
 tab_input_format::on_stretch_changed(wxCommandEvent &evt) {
-  if (input->selected_track == -1)
+  if (input->dont_copy_values_now || (input->selected_track == -1))
     return;
 
   tracks[input->selected_track]->stretch = tc_stretch->GetValue();
@@ -320,7 +400,7 @@ tab_input_format::on_stretch_changed(wxCommandEvent &evt) {
 
 void
 tab_input_format::on_aspect_ratio_changed(wxCommandEvent &evt) {
-  if (input->selected_track == -1)
+  if (input->dont_copy_values_now || (input->selected_track == -1))
     return;
 
   tracks[input->selected_track]->aspect_ratio =
@@ -329,16 +409,33 @@ tab_input_format::on_aspect_ratio_changed(wxCommandEvent &evt) {
 
 void
 tab_input_format::on_fourcc_changed(wxCommandEvent &evt) {
-  if (input->selected_track == -1)
+  if (input->dont_copy_values_now || (input->selected_track == -1))
     return;
 
   tracks[input->selected_track]->fourcc =
     cob_fourcc->GetStringSelection();
 }
 
+void
+tab_input_format::on_fps_changed(wxCommandEvent &evt) {
+  if (input->dont_copy_values_now || (input->selected_track == -1))
+    return;
+
+  tracks[input->selected_track]->fps = cob_fps->GetStringSelection();
+}
+
+void
+tab_input_format::on_nalu_size_length_changed(wxCommandEvent &evt) {
+  if (input->dont_copy_values_now || (input->selected_track == -1))
+    return;
+
+  tracks[input->selected_track]->nalu_size_length =
+    cob_nalu_size_length->GetSelection() + 2;
+}
+
 void
 tab_input_format::on_stereo_mode_changed(wxCommandEvent &evt) {
-  if (input->selected_track == -1)
+  if (input->dont_copy_values_now || (input->selected_track == -1))
     return;
 
   tracks[input->selected_track]->stereo_mode =
@@ -347,7 +444,7 @@ tab_input_format::on_stereo_mode_changed(wxCommandEvent &evt) {
 
 void
 tab_input_format::on_compression_selected(wxCommandEvent &evt) {
-  if (input->selected_track == -1)
+  if (input->dont_copy_values_now || (input->selected_track == -1))
     return;
 
   tracks[input->selected_track]->compression =
@@ -356,7 +453,7 @@ tab_input_format::on_compression_selected(wxCommandEvent &evt) {
 
 void
 tab_input_format::on_display_width_changed(wxCommandEvent &evt) {
-  if (input->selected_track == -1)
+  if (input->dont_copy_values_now || (input->selected_track == -1))
     return;
 
   tracks[input->selected_track]->dwidth = tc_display_width->GetValue();
@@ -364,7 +461,7 @@ tab_input_format::on_display_width_changed(wxCommandEvent &evt) {
 
 void
 tab_input_format::on_display_height_changed(wxCommandEvent &evt) {
-  if (input->selected_track == -1)
+  if (input->dont_copy_values_now || (input->selected_track == -1))
     return;
 
   tracks[input->selected_track]->dheight = tc_display_height->GetValue();
@@ -374,7 +471,7 @@ void
 tab_input_format::on_aspect_ratio_selected(wxCommandEvent &evt) {
   mmg_track_t *track;
 
-  if (input->selected_track == -1)
+  if (input->dont_copy_values_now || (input->selected_track == -1))
     return;
 
   track = tracks[input->selected_track];
@@ -386,7 +483,7 @@ void
 tab_input_format::on_display_dimensions_selected(wxCommandEvent &evt) {
   mmg_track_t *track;
 
-  if (input->selected_track == -1)
+  if (input->dont_copy_values_now || (input->selected_track == -1))
     return;
 
   track = tracks[input->selected_track];
@@ -400,6 +497,9 @@ BEGIN_EVENT_TABLE(tab_input_format, wxPanel)
   EVT_COMBOBOX(ID_CB_SUBTITLECHARSET, tab_input_format::on_subcharset_selected)
   EVT_COMBOBOX(ID_CB_ASPECTRATIO, tab_input_format::on_aspect_ratio_changed)
   EVT_COMBOBOX(ID_CB_FOURCC, tab_input_format::on_fourcc_changed)
+  EVT_COMBOBOX(ID_CB_FPS, tab_input_format::on_fps_changed)
+  EVT_COMBOBOX(ID_CB_NALU_SIZE_LENGTH,
+               tab_input_format::on_nalu_size_length_changed)
   EVT_COMBOBOX(ID_CB_STEREO_MODE, tab_input_format::on_stereo_mode_changed)
   EVT_TEXT(ID_CB_SUBTITLECHARSET, tab_input_format::on_subcharset_selected)
   EVT_TEXT(ID_TC_DELAY, tab_input_format::on_delay_changed)
