@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: p_aac.cpp 3358 2006-11-10 18:23:51Z mosu $
+   $Id: p_aac.cpp 3464 2007-01-13 17:41:08Z mosu $
 
    AAC output module
 
@@ -84,7 +84,7 @@ aac_packetizer_c::get_aac_packet(unsigned long *header,
 
   bytes_skipped += pos;
   if (verbose && (0 < bytes_skipped))
-    mxwarn(FMT_TID "skipping %lld bytes (no valid AAC header "
+    mxwarn(FMT_TID "skipping " LLD " bytes (no valid AAC header "
            "found). This might make audio/video go out of sync, but this "
            "stream is damaged.\n", ti.fname.c_str(), (int64_t)0,
            bytes_skipped);
