@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: p_ac3.cpp 3209 2006-04-13 12:31:46Z mosu $
+   $Id: p_ac3.cpp 3393 2006-12-28 15:25:22Z mosu $
 
    AC3 output module
 
@@ -93,7 +93,7 @@ ac3_packetizer_c::get_ac3_packet(unsigned long *header,
     }
     if (!warning_printed)
       mxwarn("The AC3 track " LLD " from '%s' contained " LLD " bytes of "
-             "non-AC3 data at the beginning which were skipped. The audio/"
+             "non-AC3 data which were skipped. The audio/"
              "video synchronization may have been lost.\n", ti.id,
              ti.fname.c_str(), bytes_skipped);
     byte_buffer.remove(pos);
