@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: p_mp3.cpp 3209 2006-04-13 12:31:46Z mosu $
+   $Id: p_mp3.cpp 3393 2006-12-28 15:25:22Z mosu $
 
    MP3 output module
 
@@ -58,15 +58,15 @@ mp3_packetizer_c::handle_garbage(int64_t bytes) {
                                    !(ti.avi_block_align % 576));
     if (offset != -1) {
       mxinfo("The MPEG audio track " LLD " from '%s' contained " LLD " bytes "
-             "of garbage at the beginning. This corresponds to a delay of "
-             "" LLD "ms. This delay will be used instead of the garbage data."
+             "of non-MP3 data at the beginning. This corresponds to a delay of"
+             " " LLD "ms. This delay will be used instead of the garbage data."
              "\n", ti.id, ti.fname.c_str(), bytes, offset / 1000000);
       warning_printed = true;
     }
   }
   if (!warning_printed)
     mxwarn("The MPEG audio track " LLD " from '%s' contained " LLD " bytes of "
-           "garbage at the beginning which were skipped. The audio/video "
+           "non-MP3 data which were skipped. The audio/video "
            "synchronization may have been lost.\n", ti.id, ti.fname.c_str(),
            bytes);
 }
