@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: p_mp3.h 2932 2005-05-25 21:31:26Z mosu $
+   $Id: p_mp3.h 3209 2006-04-13 12:31:46Z mosu $
 
    class definition for the MP3 output module
 
@@ -25,7 +25,7 @@
 
 class mp3_packetizer_c: public generic_packetizer_c {
 private:
-  int64_t bytes_output, packetno;
+  int64_t bytes_output, packetno, bytes_skipped;
   int samples_per_sec, channels, spf;
   byte_buffer_c byte_buffer;
   bool codec_id_set, valid_headers_found;
