@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: p_video.cpp 3146 2005-11-16 21:50:58Z mosu $
+   $Id: p_video.cpp 3469 2007-01-15 19:02:18Z mosu $
 
    video output module
 
@@ -66,46 +66,6 @@ video_packetizer_c::check_fourcc() {
            ti.fourcc.c_str(), 4);
     set_codec_private(ti.private_data, ti.private_size);
   }
-
-  if (!hack_engaged(ENGAGE_ALLOW_AVC_IN_VFW_MODE) &&
-      (hcodec_id == MKV_V_MSCOMP) &&
-      (ti.private_data != NULL) &&
-      (ti.private_size >= sizeof(alBITMAPINFOHEADER))) {
-    const char *avc_fourccs[] = {
-      "h264", "x264", "avc1", NULL
-    };
-    const char *fourcc;
-    int i;
-
-    fourcc = (const char *)
-      &((alBITMAPINFOHEADER *)ti.private_data)->bi_compression;
-    for (i = 0; NULL != avc_fourccs[i]; ++i)
-      if (!strncasecmp(fourcc, avc_fourccs[i], 4))
-        mxerror(FMT_TID "You are trying to put AVC/h.264 video from an AVI "
-                "or a similar VfW (Video for Windows) compatible source into "
-                "a Matroska file in the so-called 'VfW compatibility mode'. "
-                "Please note that this is not the official way to store "
-                "AVC/h.264 video in Matroska. Therefore proper playback of "
-                "such files cannot be guaranteed, and we strongly urge you "
-                "to use the native Matroska-mode.\n"
-                "At the moment mkvmerge does not support converting from "
-                "VfW-mode AVC/h.264 tracks to native Matroska-mode AVC/h.264 "
-                "tracks. You can, however, first import the video track "
-                "into a MP4 file with e.g. 'MP4Box' (use Google). Then "
-                "you can use mkvmerge and put the video into a Matroska "
-                "file.\n"
-                "If you're trying to get AVC/h.264 from an AVI into Matroska "
-                "then the commands you need to use are:\n"
-                " avi2raw avc.avi raw.264\n"
-                " mp4box -fps 23.976 -add raw.264 avc.mp4\n\n"
-                "If you really know what you are doing then you can force "
-                "mkvmerge to put this AVC/h.264 track into a Matroska file "
-                "even in VfW mode if you add '--engage allow_avc_in_vfw_mode' "
-                "to the command line. You can do that in mmg with the "
-                "'Add command line options' menu entry in the 'Muxing' "
-                "menu.\n",
-                ti.fname.c_str(), (int64_t)ti.id);
-  }
 }
 
 void
@@ -191,7 +151,7 @@ video_packetizer_c::can_connect_to(generic_packetizer_c *src,
     error_message = mxsprintf("The codec's private data does not match "
                               "(lengths: %d and %d).", ti.private_size,
                               vsrc->ti.private_size);
-    return CAN_CONNECT_NO_PARAMETERS;
+    return CAN_CONNECT_MAYBE_CODECPRIVATE;
   }
   return CAN_CONNECT_YES;
 }
@@ -223,6 +183,62 @@ mpeg1_2_video_packetizer_c(generic_reader_c *_reader,
     aspect_ratio_extracted = true;
 
   timecode_factory_application_mode = TFA_SHORT_QUEUEING;
+  if (hack_engaged(ENGAGE_USE_CODEC_STATE))
+      parser.SeparateSequenceHeaders();
+}
+
+int
+mpeg1_2_video_packetizer_c::process_framed(packet_cptr packet) {
+  if (0 == packet->data->get_size())
+    return FILE_STATUS_MOREDATA;
+
+  if (!hack_engaged(ENGAGE_USE_CODEC_STATE) ||
+      (4 > packet->data->get_size()))
+    return video_packetizer_c::process(packet);
+
+  unsigned char *buf = packet->data->get();
+  int pos = 4, marker, size = packet->data->get_size();
+
+  marker = get_uint32_be(buf);
+  while ((pos < size) && (MPEGVIDEO_SEQUENCE_START_CODE != marker)) {
+    marker <<= 8;
+    marker |= buf[pos];
+    ++pos;
+  }
+
+  if ((MPEGVIDEO_SEQUENCE_START_CODE != marker) ||
+      ((pos + 4) >= size))
+    return video_packetizer_c::process(packet);
+
+  int start = pos - 4;
+  marker = get_uint32_be(&buf[pos]);
+  pos += 4;
+
+  while ((pos < size) &&
+         ((MPEGVIDEO_EXT_START_CODE == marker) ||
+          (0x00000100 != (marker & 0xffffff00)))) {
+    marker <<= 8;
+    marker |= buf[pos];
+    ++pos;
+  }
+
+  if (pos >= size)
+    return video_packetizer_c::process(packet);
+
+  pos -= 4;
+  int sh_size = pos - start;
+
+  if ((NULL == seq_hdr.get()) ||
+      (sh_size != seq_hdr->get_size()) ||
+      memcmp(&buf[start], seq_hdr->get(), sh_size)) {
+    seq_hdr = clone_memory(&buf[start], sh_size);
+    packet->codec_state = clone_memory(&buf[start], sh_size);
+  }
+
+  memmove(&buf[start], &buf[pos], size - pos);
+  packet->data->set_size(size - sh_size);
+
+  return video_packetizer_c::process(packet);
 }
 
 int
@@ -236,12 +252,8 @@ mpeg1_2_video_packetizer_c::process(packet_cptr packet) {
   if (!aspect_ratio_extracted)
     extract_aspect_ratio(packet->data->get(), packet->data->get_size());
 
-  if (framed) {
-    if (0 != packet->data->get_size())
-      return video_packetizer_c::process(packet);
-    else
-      return FILE_STATUS_MOREDATA;
-  }
+  if (framed)
+    return process_framed(packet);
 
   state = parser.GetState();
   if ((state == MPV_PARSER_STATE_EOS) ||
@@ -271,15 +283,23 @@ mpeg1_2_video_packetizer_c::process(packet_cptr packet) {
       if (frame == NULL)
         break;
 
-      if (hcodec_private == NULL)
-        create_private_data();
-
       packet_t *new_packet =
         new packet_t(new memory_c(frame->data, frame->size, true),
                      frame->timecode, frame->duration,
                      frame->firstRef, frame->secondRef);
       new_packet->time_factor =
         MPEG2_PICTURE_TYPE_FRAME == frame->pictureStructure ? 1 : 2;
+
+      if ((NULL != frame->seqHdrData) &&
+          ((NULL == seq_hdr.get()) ||
+           (frame->seqHdrDataSize != seq_hdr->get_size()) ||
+           memcmp(frame->seqHdrData, seq_hdr->get(), frame->seqHdrDataSize))) {
+        seq_hdr = memory_cptr(new memory_c(frame->seqHdrData,
+                                           frame->seqHdrDataSize, true));
+        frame->seqHdrData = NULL;
+        new_packet->codec_state = clone_memory(seq_hdr);
+      }
+
       video_packetizer_c::process(packet_cptr(new_packet));
       frame->data = NULL;
       delete frame;
@@ -304,9 +324,9 @@ mpeg1_2_video_packetizer_c::extract_fps(const unsigned char *buffer,
                                         int size) {
   int idx;
 
-  idx = mpeg1_2_extract_fps_idx(buffer, size);
+  idx = mpeg1_2::extract_fps_idx(buffer, size);
   if (idx >= 0) {
-    fps = mpeg1_2_get_fps(idx);
+    fps = mpeg1_2::get_fps(idx);
     if (fps > 0) {
       set_track_default_duration((int64_t)(1000000000.0 / fps));
       rerender_track_headers();
@@ -323,7 +343,7 @@ mpeg1_2_video_packetizer_c::extract_aspect_ratio(const unsigned char *buffer,
   if (ti.aspect_ratio_given || ti.display_dimensions_given)
     return;
 
-  if (mpeg1_2_extract_ar(buffer, size, ar)) {
+  if (mpeg1_2::extract_ar(buffer, size, ar)) {
     ti.display_dimensions_given = true;
     if ((ar <= 0) || (ar == 1))
       set_video_display_width(width);
@@ -404,8 +424,8 @@ mpeg4_p2_video_packetizer_c::process_non_native(packet_cptr packet) {
   if (NULL == ti.private_data) {
     memory_c *config_data;
 
-    config_data = mpeg4_p2_parse_config_data(packet->data->get(),
-                                             packet->data->get_size());
+    config_data = mpeg4::p2::parse_config_data(packet->data->get(),
+                                               packet->data->get_size());
     if (NULL != config_data) {
       ti.private_data = (unsigned char *)safememdup(config_data->get(),
                                                     config_data->get_size());
@@ -421,8 +441,8 @@ mpeg4_p2_video_packetizer_c::process_non_native(packet_cptr packet) {
               "native mode.\n");
   }
 
-  mpeg4_p2_find_frame_types(packet->data->get(), packet->data->get_size(),
-                            frames);
+  mpeg4::p2::find_frame_types(packet->data->get(), packet->data->get_size(),
+                              frames);
 
   // Add a timecode and a duration if they've been given.
   if (-1 != packet->timecode)
@@ -434,7 +454,7 @@ mpeg4_p2_video_packetizer_c::process_non_native(packet_cptr packet) {
   if (-1 != packet->duration)
     available_durations.push_back(packet->duration);
 
-  foreach(frame, frames) {
+  mxforeach(frame, frames) {
     // Maybe we can flush queued frames now. But only if we don't have
     // a B frame.
     if (FRAME_TYPE_B != frame->type)
@@ -670,7 +690,7 @@ mpeg4_p2_video_packetizer_c::extract_aspect_ratio(const unsigned char *buffer,
     return;
   }
 
-  if (mpeg4_p2_extract_par(buffer, size, num, den)) {
+  if (mpeg4::p2::extract_par(buffer, size, num, den)) {
     aspect_ratio_extracted = true;
     ti.aspect_ratio_given = true;
     ti.aspect_ratio = (float)hvideo_pixel_width /
@@ -690,7 +710,7 @@ mpeg4_p2_video_packetizer_c::extract_size(const unsigned char *buffer,
                                           int size) {
   uint32_t width, height;
 
-  if (mpeg4_p2_extract_size(buffer, size, width, height)) {
+  if (mpeg4::p2::extract_size(buffer, size, width, height)) {
     size_extracted = true;
     if (!reader->appending &&
         ((width != hvideo_pixel_width) || (height != hvideo_pixel_height))) {
@@ -726,6 +746,8 @@ mpeg4_p10_video_packetizer_c(generic_reader_c *_reader,
                              track_info_c &_ti):
   video_packetizer_c(_reader, MKV_V_MPEG4_AVC, _fps, _width, _height, _ti) {
 
+  relaxed_timecode_checking = true;
+
   if ((ti.private_data != NULL) && (ti.private_size > 0))
     extract_aspect_ratio();
 }
@@ -736,7 +758,7 @@ mpeg4_p10_video_packetizer_c::extract_aspect_ratio() {
   unsigned char *priv;
 
   priv = ti.private_data;
-  if (mpeg4_p10_extract_par(ti.private_data, ti.private_size, num, den) &&
+  if (mpeg4::p10::extract_par(ti.private_data, ti.private_size, num, den) &&
       (0 != num) && (0 != den)) {
     if (!ti.aspect_ratio_given && !ti.display_dimensions_given) {
       double par = (double)num / (double)den;
@@ -798,7 +820,7 @@ mpeg4_p10_video_packetizer_c::can_connect_to(generic_packetizer_c *src,
     error_message = mxsprintf("The codec's private data does not match."
                               "Both have the same length (%d) but different "
                               "content.", ti.private_size);
-    return CAN_CONNECT_NO_PARAMETERS;
+    return CAN_CONNECT_MAYBE_CODECPRIVATE;
   }
 
   return CAN_CONNECT_YES;
