@@ -11,7 +11,7 @@ class T_205X_cuesheets < Test
     sys("../src/mkvextract cuesheet #{tmp}-src --no-variable-data &> #{tmp}")
     hash = hash_tmp(false)
     sys("../src/mkvextract tracks #{tmp}-src --no-variable-data " +
-         "--cuesheet 1:#{tmp} > /tmp/a")
+         "--cuesheet 1:#{tmp} > /dev/null")
     hash += "-" + hash_tmp("#{tmp}.cue")
     unlink_tmp_files
     return hash
